package com.bocloud.cos.booter.config;

import com.megatron.common.utils.MapTools;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import java.util.Map;

/**
 * rabbitmq配置类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年3月30日
 */
@Configuration
@ConfigurationProperties(prefix = "spring.rabbitmq")
public class RabbitMQConfiguration {

    private final Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");
    private final ConnectionFactory connectionFactory;

    @Autowired
    public RabbitMQConfiguration(ConnectionFactory connectionFactory) {
        this.connectionFactory = connectionFactory;
    }

    /**
     * 声明默认的交换
     *
     * @return
     */
    @Bean
    public DirectExchange deafultExchange() {
        return new DirectExchange("bocloud.direct.exchange", true, true);
    }

    /**
     * 资源编排
     *
     * @return
     */
    @Bean
    public Queue declareRosChargingQueue() {
        return new Queue("ros.charging.queue", true, false, false, arguments);
    }

    /**
     * 声明资源结果队列
     *
     * @return
     */
    @Bean
    public Queue declareResourceQueue() {
        return new Queue("resource.result.queue", true, false, false, arguments);
    }

    /**
     * 声明申请处理队列
     *
     * @return
     */
    @Bean
    public Queue declareFlowQueue() {
        return new Queue("bocloud.flow.queue", true, false, false, arguments);
    }

    /**
     * 声明授权队列
     *
     * @return
     */
    @Bean
    public Queue declareQuotaQueue() {
        return new Queue("tenant.grant.queue", true, false, false, arguments);
    }

    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }

    @Bean
    public Binding bindingRosChargingQueue() {
        return BindingBuilder.bind(declareRosChargingQueue()).to(deafultExchange()).withQueueName();
    }


    /**
     * 绑定资源结果队列
     *
     * @return
     */
    @Bean
    public Binding bindingResourceQueue() {
        return BindingBuilder.bind(declareResourceQueue()).to(deafultExchange()).withQueueName();
    }


    /**
     * 绑定流程处理队列
     *
     * @return
     */
    @Bean
    public Binding bindingFlowQueue() {
        return BindingBuilder.bind(declareFlowQueue()).to(deafultExchange()).withQueueName();
    }

    /**
     * 绑定配额变更队列
     *
     * @return
     */
    @Bean
    public Binding bindingQuotaQueue() {
        return BindingBuilder.bind(declareQuotaQueue()).to(deafultExchange()).withQueueName();
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitTemplate amqpTemplate() {
        RabbitTemplate amqpTemplate = new RabbitTemplate(connectionFactory);
        amqpTemplate.setMessageConverter(messageConverter());
        amqpTemplate.setExchange(deafultExchange().getName());
        return amqpTemplate;
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitAdmin rabbitAdmin() {
        return new RabbitAdmin(connectionFactory);
    }
}
