package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceCategory;
import com.bocloud.cos.intf.CloudServiceService;
import com.bocloud.cos.intf.ServiceCategoryService;
import com.bocloud.cos.model.BatchServiceFlowBean;
import com.bocloud.cos.model.ServiceFlowBean;
import com.bocloud.cos.params.CloudServiceActionParams;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;

/**
 * @author tyl
 */
@RestController
@RequestMapping("/v1/cloud/services")
@Tag(name = "云服务管理")
public class CloudServiceController {

    private final CloudServiceService cloudServiceService;
    private final ServiceCategoryService serviceCategoryService;

    public CloudServiceController(CloudServiceService cloudServiceService,
                                  ServiceCategoryService serviceCategoryService) {
        this.cloudServiceService = cloudServiceService;
        this.serviceCategoryService = serviceCategoryService;
    }

    /**
     * 获取服务列表
     *
     * @param pager
     * @param isAll 是否全部显示
     * @return
     */
    @Operation(summary = "获取服务列表")
    @GetMapping
    public GeneralResult<GridBean<CloudService>> list(Pager pager, @RequestParam(required = false) Boolean isAll) {
        isAll = Optional.ofNullable(isAll).orElse(false);
        return cloudServiceService.list(pager, isAll);
    }

    /**
     * 绑定服务
     *
     * @return {@link Result}
     */
    @Operation(summary = "服务绑定")
    @PutMapping("binding/plugins")
    public Result BindingService(@RequestParam("pluginCode") String pluginCode,
                                 @RequestParam("cloudServiceIds") List<Long> cloudServiceIds) {
        cloudServiceService.bindingService(pluginCode, cloudServiceIds);
        return Result.SUCCESS("绑定服务成功");
    }

    /**
     * 获取服务列表condition查询方式
     *
     * @param condition
     * @return
     */
    @Operation(summary = "获取服务列表")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.condition(condition, context);
    }

    /**
     * 添加云服务
     *
     * @param cloudService
     * @param context
     * @return
     */
    @Operation(summary = "添加云服务")
    @PostMapping
    public Result create(@RequestBody CloudService cloudService,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        cloudService.setCreatorId(context.getTarget());
        cloudService.setMenderId(context.getTarget());
        return cloudServiceService.create(cloudService);
    }

    /**
     * 修改服务
     *
     * @param id
     * @param cloudService
     * @param context
     * @return
     */
    @Operation(summary = "修改云服务信息")
    @PutMapping("/{id}")
    public Result modify(@RequestBody CloudService cloudService,
                         @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        cloudService.setId(id);
        return cloudServiceService.modify(cloudService, context.getTarget());
    }

    /**
     * 移除服务
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(summary = "移除服务")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.remove(id, context.getTarget());
    }

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    @Operation(summary = "查询服务详情")
    @GetMapping("/{id}")
    public GeneralResult<CloudService> detail(@PathVariable(value = Common.ID) Long id) {
        return cloudServiceService.detail(id);
    }

    /**
     * 服务操作
     *
     * @param id           服务id
     * @param actionParams 操作参数
     * @param context      请求对象
     * @return
     */
    @Operation(summary = "服务操作")
    @PatchMapping(value = "/{id}")
    public Result action(@PathVariable(value = Common.ID) Long id,
                         @RequestBody CloudServiceActionParams actionParams,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (actionParams.getAction()) {
            case UNPUBLISHED:
            case PUBLISHED:
                return cloudServiceService.updateStatus(id, actionParams.getAction().name(), context.getTarget());
            default:
                return new Result(false, "暂不支持该类操作");
        }
    }


    /**
     * 服务批量操作
     *
     * @param actionParams 操作参数
     * @param context      请求对象
     * @return
     */
    @Operation(summary = "服务操作")
    @PatchMapping
    public Result batchAction(@RequestBody CloudServiceActionParams actionParams,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (actionParams.getAction()) {
            case UNPUBLISHED:
                return cloudServiceService.batchUnpublished(actionParams.getIds(), context.getTarget());
            case PUBLISHED:
                return cloudServiceService.batchPublished(actionParams.getIds(), context.getTarget());
            default:
                return new Result(false, "暂不支持该类操作");
        }
    }

    /**
     * 根据code query
     *
     * @param code
     * @return
     */
    @Operation(summary = "根据code查询服务类型")
    @GetMapping("/{code}/categories")
    public GeneralResult<List<ServiceCategory>> listByCode(@PathVariable(value = Common.CODE) String code) {
        return serviceCategoryService.listByCode(code);
    }

    /**
     * 获取租户授权服务的平台类型
     *
     * @param catalogId
     * @return
     */
    @Operation(summary = "获取租户授权服务的平台类型")
    @GetMapping("/vendors")
    public GeneralResult<List<String>> vendors(@RequestParam Long catalogId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.vendors(catalogId, context.getTenant());
    }

    /**
     * 查询租户指定平台下的服务
     *
     * @param catalogId
     * @return
     */
    @Operation(summary = "获取租户指定平台下的服务")
    @GetMapping("/vendors/{vendor}")
    public GeneralResult<List<CloudService>> vendorService(@PathVariable(value = "vendor") String vendor, @RequestParam Long catalogId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.vendorService(vendor, catalogId, context.getTenant());
    }


    /**
     * 服务配置流程
     *
     * @param serviceId
     * @param serviceFlowBeans
     * @return
     */
    @Operation(summary = "服务配置流程")
    @PostMapping("/{id}/flows")
    public Result configFlow(@PathVariable(value = "id") Long serviceId, @RequestBody List<ServiceFlowBean> serviceFlowBeans, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.configFlow(serviceId, serviceFlowBeans, context);
    }


    /**
     * 服务批量配置流程
     *
     * @param batchServiceFlowBean
     * @return
     */
    @Operation(summary = "服务批量配置流程")
    @PostMapping("/flows")
    public Result batchConfigFlow(@RequestBody BatchServiceFlowBean batchServiceFlowBean, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.batchConfigFlow(batchServiceFlowBean, context);
    }

}
