package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceTenant;
import com.bocloud.cos.intf.CloudServiceService;
import com.bocloud.cos.intf.ServiceTenantService;
import com.bocloud.cos.model.GrantBean;
import com.bocloud.cos.params.GrantBatchParams;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 服务授权
 *
 * @author Administrator
 */
@RestController
@RequestMapping("/v1/services")
@Tag(name = "服务授权管理")
public class ServiceTenantController {

    private final ServiceTenantService serviceTenantService;
    private final CloudServiceService cloudServiceService;

    public ServiceTenantController(ServiceTenantService serviceTenantService, CloudServiceService cloudServiceService) {
        this.serviceTenantService = serviceTenantService;
        this.cloudServiceService = cloudServiceService;
    }

    /**
     * 查询租户授权服务列表
     *
     * @param code
     * @param suffix
     * @param context
     * @return
     */
    @GetMapping("/tenants")
    @Operation(summary = "查询租户授权服务列表")
    public GeneralResult<List<CloudService>> tenantServices(@RequestParam(value = Common.CODE) String code,
                                                            @RequestParam(value = "suffix", required = false) String suffix,
                                                            @Value(value = Common.REQ_CONTEXT) RequestContext context) {
        return serviceTenantService.listByTenant(code, suffix, context);
    }

    /**
     * 租户权限详细
     *
     * @param id 租户id
     * @return
     */
    @GetMapping("/{id}/tenants")
    @Operation(summary = "服务权限授众")
    public GeneralResult<List<ServiceTenant>> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceTenantService.detail(id);
    }

    /**
     * 服务目录批量授权与批量收权
     *
     * @param params 批量授权对象
     * @return
     */
    @PostMapping("/tenants/grantbatch")
    @Operation(summary = "服务目录批量授权收权")
    public Result grantBatch(@RequestBody GrantBatchParams params,
                             @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return serviceTenantService.grantBatch(params.getServiceIds(), params.getTenantIds(), params.getType(), requestContext);
    }

    /**
     * 单一租户同时授权多个服务
     *
     * @param id
     * @param grantBean
     * @param requestContext
     * @return
     */
    @PostMapping("/tenants/{id}/grant")
    @Operation(summary = "租户授权服务")
    public Result grantService(@PathVariable(value = Common.ID) Long id,
                               @RequestBody GrantBean grantBean,
                               @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return serviceTenantService.grantService(id, grantBean, requestContext);
    }

    /**
     * 单一服务同时授权多个租户
     *
     * @param serviceId
     * @param tenantIds 租户id集合
     * @return
     */
    @PostMapping("/{id}/grant")
    @Operation(summary = "服务授权多个租户")
    public Result grantTenant(@PathVariable(value = "id") Long serviceId,
                              @RequestBody List<Long> tenantIds,
                              @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return serviceTenantService.grantTenant(serviceId, tenantIds, requestContext);
    }

    /**
     * 根据租户id删除租户服务关系表数据
     *
     * @param id             租户id
     * @param requestContext
     * @return
     */
    @Operation(summary = "删除租户服务授权")
    @DeleteMapping("/tenants/{id}")
    public Result deleteByTenant(@PathVariable(value = Common.ID) Long id,
                                 @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return serviceTenantService.deleteByTenant(id, requestContext);
    }

    /**
     * 查询租户下的服务
     *
     * @param id 租户id
     * @return
     */
    @GetMapping("/tenants/{id}/services")
    @Operation(summary = "查询租户下服务列表")
    public GeneralResult<List<CloudService>> getTenantService(@PathVariable(value = Common.ID) Long id,
                                                              @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return cloudServiceService.getTenantService(id, requestContext);
    }

    /**
     * 查询租户下可授权的服务
     *
     * @param id 租户id
     * @return
     */
    @GetMapping("/tenants/{id}/able")
    @Operation(summary = "查询租户下可授权的服务列表")
    public Result getAbleServices(@PathVariable(value = Common.ID) Long id,
                                  @RequestParam(value = "event", required = false) String event,
                                  @RequestParam(value = "quotaAble", required = false) Boolean quotaAble,
                                  @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return cloudServiceService.getAbleServices(id, event, quotaAble, requestContext);
    }

    /**
     * 根据code获取cloudService
     *
     * @param codes
     * @return
     */
    @GetMapping
    @Operation(summary = "根据code获取服务")
    public GeneralResult<List<CloudService>> listByCode(@RequestParam(value = "codes") List<String> codes) {
        return cloudServiceService.listByCode(codes);
    }
}
