package com.bocloud.cos.repository;

import com.bocloud.cos.annotations.Provider;
import com.bocloud.cos.model.Butler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.JarURLConnection;
import java.net.URL;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

@Repository
public class ProviderRepository {

    private static Logger logger = LoggerFactory.getLogger(ProviderRepository.class);
    private static Map<String, Class<?>> providerMap = new ConcurrentHashMap();

    public ProviderRepository() {
    }

    public Object getProvider(String code, Class<?> clazz, Butler butler) {
        try {
            Class<?> providerClass = this.getProviderClass(code, clazz);
            Constructor<?> constructor = providerClass.getDeclaredConstructor(Butler.class);
            return constructor.newInstance(butler);
        } catch (NoSuchMethodException var6) {
            logger.error("init provider {} of {} NoSuchMethodException!", new Object[]{clazz, code, var6});
        } catch (InstantiationException var7) {
            logger.error("init provider {} of {} InstantiationException!", new Object[]{clazz, code, var7});
        } catch (InvocationTargetException var8) {
            logger.error("init provider {} of {} InvocationTargetException!", new Object[]{clazz, code, var8});
        } catch (IllegalAccessException var9) {
            logger.error("init provider {} of {} IllegalAccessException!", new Object[]{clazz, code, var9});
        } catch (Exception var10) {
            logger.error("get specific provider class {} of {} exception", new Object[]{clazz, code, var10});
        }

        return null;
    }


    private List<Class<?>> getProviderClasses(String code, Class<?> clazz) {
        List<Class<?>> allClass = this.getAllClass(clazz.getPackage().getName());
        Set<Class<?>> fixClass = new HashSet();
        Iterator var5 = allClass.iterator();

        while (var5.hasNext()) {
            Class<?> providerClass = (Class) var5.next();
            if (clazz.isAssignableFrom(providerClass) && !clazz.equals(providerClass) && providerClass.isAnnotationPresent(Provider.class)) {
                Provider provider = providerClass.getAnnotation(Provider.class);
                if (provider.value().equalsIgnoreCase(code)) {
                    fixClass.add(providerClass);
                }
            }
        }
        return new ArrayList<>(fixClass);
    }

    public synchronized Class<?> getProviderClass(String code, Class<?> clazz) throws Exception {
        if (providerMap.containsKey(buildKey(code, clazz))) {
            return providerMap.get(buildKey(code, clazz));
        } else {
            List<Class<?>> fixClass = this.getProviderClasses(code, clazz);
            if (fixClass.isEmpty()) {
                throw new Exception("class not found");
            } else if (fixClass.size() > 1) {
                throw new Exception("more than one class found!");
            } else {
                providerMap.put(buildKey(code, clazz), fixClass.get(0));
                return fixClass.get(0);
            }
        }
    }

    private List<Class<?>> getAllClass(String packageName) {
        List<Class<?>> list = new ArrayList();
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        String path = packageName.replace(".", "/");

        try {
            Enumeration enumeration = classLoader.getResources(path);

            while (enumeration.hasMoreElements()) {
                URL url = (URL) enumeration.nextElement();
                if ("jar".equals(url.getProtocol())) {
                    String jarPath = url.getPath();
                    if (jarPath.contains("provider")) {
                        JarFile jarFile = ((JarURLConnection) url.openConnection()).getJarFile();
                        list.addAll(findClassFromJar(jarFile, path));
                    }
                } else {
                    list.addAll(this.findClassFromFile(new File(url.getFile()), packageName));
                }
            }
        } catch (IOException var9) {
            logger.error("get all class of package {} IOException!", packageName, var9);
        }

        return list;
    }

    public static List<Class<?>> findClassFromJar(JarFile jarFile, String path) {
        List<Class<?>> classList = new ArrayList();
        String packageName = path.replace(".", "/");

        try {
            Enumeration<JarEntry> entries = jarFile.entries();
            String entryName;

            while (entries.hasMoreElements()) {
                JarEntry entry = entries.nextElement();
                if (!entry.isDirectory()) {
                    entryName = entry.getName();
                    if (entryName.charAt(0) == '/') {
                        entryName = entryName.substring(1);
                    }

                    if (entryName.startsWith(packageName) && entryName.endsWith(".class")) {
                        String className = entryName.substring(0, entryName.length() - 6);
                        Class<?> clazz = loadClass(className.replace("/", "."));
                        if (null != clazz) {
                            classList.add(clazz);
                        }
                    }
                }
            }
        } catch (Exception var17) {
            logger.error("get class from jar {} error!", jarFile.getName(), var17);
        } finally {
            if (null != jarFile) {
                try {
                    jarFile.close();
                } catch (Exception var16) {
                    logger.error("close jar {} error!", jarFile.getName(), var16);
                }
            }

        }

        return classList;
    }

    private List<Class<?>> findClassFromFile(File file, String packageName) {
        List<Class<?>> list = new ArrayList();
        if (!file.exists()) {
            return list;
        } else {
            File[] files = file.listFiles();
            File[] var5 = files;
            int var6 = files.length;

            for (int var7 = 0; var7 < var6; ++var7) {
                File classFile = var5[var7];
                if (classFile.isDirectory()) {
                    if (!classFile.getName().contains(".")) {
                        List<Class<?>> arrayList = this.findClassFromFile(classFile, packageName + "." + classFile.getName());
                        list.addAll(arrayList);
                    }
                } else if (classFile.getName().endsWith(".class")) {
                    try {
                        list.add(Class.forName(packageName + '.' + classFile.getName().substring(0, classFile.getName().length() - 6)));
                    } catch (ClassNotFoundException var10) {
                        logger.error("get class of {} occur ClassNotFoundException!", file, var10);
                    }
                }
            }

            return list;
        }
    }

    private static String buildKey(String code, Class<?> clazz) {
        return code + ":" + clazz.getName();
    }

    private static Class<?> loadClass(String fullClassName) {
        try {
            return Thread.currentThread().getContextClassLoader().loadClass(fullClassName);
        } catch (ClassNotFoundException var2) {
            logger.error("load class of {} occur ClassNotFoundException!", var2);
            return null;
        }
    }
}
