package com.bocloud.cos.provider.huawei;

import com.megatron.common.utils.ListTool;
import com.bocloud.cos.driver.SuperProvider;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.model.HuaweiIndex;
import com.huaweicloud.sdk.bss.v2.BssClient;
import com.huaweicloud.sdk.bss.v2.region.BssRegion;
import com.huaweicloud.sdk.core.ClientBuilder;
import com.huaweicloud.sdk.core.auth.GlobalCredentials;
import com.huaweicloud.sdk.core.auth.ICredential;
import com.huaweicloud.sdk.core.http.HttpConfig;
import com.huaweicloud.sdk.iam.v3.IamClient;
import com.huaweicloud.sdk.iam.v3.model.KeystoneListProjectsRequest;
import com.huaweicloud.sdk.iam.v3.model.KeystoneListProjectsResponse;
import com.huaweicloud.sdk.iam.v3.model.ProjectResult;
import com.huaweicloud.sdk.iam.v3.region.IamRegion;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

@Slf4j
public class HuaweiProvider implements SuperProvider {

    private String regionId = "cn-central-220";
    private String accessKey;
    private String secretKey;
    private String endpoint;
    private HttpConfig httpConfig;

    public HuaweiProvider(Butler butler) {
        this.accessKey = butler.getAccessKey();
        this.secretKey = butler.getSecretKey();
        this.regionId = butler.getRegionId() == null ? this.regionId : butler.getRegionId();
        this.endpoint = butler.getUrl();
        this.httpConfig = HttpConfig.getDefaultHttpConfig();
        this.httpConfig.setIgnoreSSLVerification(true);
        this.httpConfig.setTimeout(50000);
        if (!StringUtils.isEmpty(butler.getProxyUri())) {
            String host = butler.getProxyUri().substring(0, butler.getProxyUri().lastIndexOf(":"));
            int port = Integer.valueOf(butler.getProxyUri().substring(butler.getProxyUri().lastIndexOf(":") + 1));
            this.httpConfig.withProxyHost(host).withProxyPort(port);
        }

    }

    public String fullEndpoint(String endpoint, HuaweiIndex.service service) {
        if (StringUtils.isEmpty(endpoint)) {
            return null;
        } else {
            String serviceName = "";
            if (HuaweiIndex.service.IAM == service) {
                serviceName = service.name().toLowerCase() + "-pub";
            } else if (HuaweiIndex.service.EIP_VPC != service && HuaweiIndex.service.ELB != service) {
                serviceName = service.name().toLowerCase();
            } else {
                serviceName = HuaweiIndex.service.VPC.name().toLowerCase();
            }

            return "https://" + serviceName + "." + this.regionId + "." + endpoint;
        }
    }

    public Object getClient(HuaweiIndex.service service) {
        ICredential globalCredential;
        switch (service) {
            case IAM:
                globalCredential = (new GlobalCredentials()).withAk(this.accessKey).withSk(this.secretKey);
                ClientBuilder<IamClient> iamBuilder = IamClient.newBuilder().withCredential(globalCredential).withHttpConfig(this.httpConfig);
                if (StringUtils.isEmpty(this.endpoint)) {
                    iamBuilder.withRegion(IamRegion.valueOf(this.regionId));
                } else {
                    iamBuilder.withEndpoint(this.fullEndpoint(this.endpoint, service));
                }

                return iamBuilder.build();
            case BSS:
                globalCredential = new GlobalCredentials().withAk(this.accessKey).withSk(this.secretKey);
                return BssClient.newBuilder().withCredential(globalCredential).withRegion(BssRegion.valueOf("cn-north-1")).withHttpConfig(this.httpConfig).build();
            default:
                return null;
        }
    }

    public String getProjectId() {
        Map<String, String> regionMap = this.regionMap();
        return regionMap.get(this.regionId);
    }

    public Map<String, String> regionMap() {
        HashMap regionMap = new HashMap();

        try {
            IamClient client = (IamClient) this.getClient(HuaweiIndex.service.IAM);
            KeystoneListProjectsRequest request = new KeystoneListProjectsRequest();
            KeystoneListProjectsResponse response = client.keystoneListProjects(request);
            if (!ListTool.isEmpty(response.getProjects())) {
                Iterator var5 = response.getProjects().iterator();

                while (var5.hasNext()) {
                    ProjectResult projectResult = (ProjectResult) var5.next();
                    regionMap.put(projectResult.getName(), projectResult.getId());
                }
            }
        } catch (Exception var7) {
            log.error("list regions error:", var7);
        }

        return regionMap;
    }

    @Override
    public boolean accessable() {
        try {
            this.getClient(HuaweiIndex.service.IAM);
            return true;
        } catch (Exception var2) {
            var2.printStackTrace();
            log.error("获取华为连接错误 ： " + var2);
            return false;
        }
    }
}
