package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.Coupon;
import com.bocloud.cos.entity.Promotion;
import com.bocloud.cos.entity.Promotion.Status;
import com.bocloud.cos.service.internal.SmsTenantService;
import com.bocloud.cos.intf.PromotionService;
import com.bocloud.cos.repository.CouponRepository;
import com.bocloud.cos.repository.PromotionRepository;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.entity.internel.Tenant;
import com.google.common.collect.Maps;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service("promotionService")
@Slf4j
public class PromotionServiceImpl implements PromotionService {

	
	@Autowired
	private PromotionRepository promotionRepository;
	@Autowired
	private CouponRepository couponRepository;
	@Autowired
	private LockFactory lockFactory;
	@Autowired
	private SmsTenantService tenantService;

	@Override
	public GeneralResult list(int page, int rows, List<Param> params, Map<String, String> sorter, RequestContext context) {
		try {
			List<Promotion> list = null;
			int total = 0;
			GridBean gridBean;
			params = null == params ? new ArrayList<>() : params;
			sorter = null == sorter ? new HashMap<>() : sorter;
			sorter.put("gmtCreate", Common.ONE);
			if (context.getCatalog().equals(RequestContext.Catalog.Manager)) {
				total = promotionRepository.count(params);
				list = promotionRepository.list(page, rows, sorter, params);
			} else {
				Long tenant = context.getTenant();
				total = promotionRepository.countTenant(params, tenant);
				list = promotionRepository.listTenant(page, rows, sorter, params, tenant);
				for (Promotion promotion : list) {
					List<Coupon> coupons = couponRepository.listByPromotionId(promotion.getId(), tenant);
					promotion.setStock(coupons.stream().filter(coupon -> coupon.getStatus().equals(Coupon.Status.unused.name())).count());
					promotion.setQuota((long) coupons.size());
				}
			}

			gridBean = GridHelper.getBean(page, rows, total, list);
			return new GeneralResult(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：{}", e);
			return new GeneralResult(false, "查询失败");
		}
	}


	private Status handleStatus(Date startDate, Date endDate) {
		Date date = new Date();
		if (date.before(startDate)) {
			return Status.not_started;
		}
		if (date.after(startDate) && date.before(endDate)) {
			return Status.progress;
		}
		if (date.after(endDate)) {
			return Status.expired;
		}
		return Status.not_started;
	}

	@Override
	public GeneralResult create(Promotion promotion, RequestContext context) {
		try {
			if (promotionRepository.existName(promotion.getName())) {
				return new GeneralResult(false, "活动名称已存在");
			}
			promotion.setStatus(handleStatus(promotion.getStartDate(), promotion.getEndDate()).name());
			promotion.setStock(null == promotion.getQuota() ? 0L : promotion.getQuota());
			promotion.setQuota(null == promotion.getQuota() ? 0L : promotion.getQuota());
			promotion.setCreatorId(context.getTarget());
			promotionRepository.save(promotion);
			return new GeneralResult(true, promotion, "创建成功");
		} catch (Exception e) {
			log.error("Get error message：{}", e);
			return new GeneralResult(false, "创建失败");
		}
	}

	@Override
	@Transactional(rollbackFor = DataBaseException.class)
	public GeneralResult modify(Promotion promotion, RequestContext context) {
		String path = Promotion.class.getSimpleName() + "_" + promotion.getId();
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			Promotion dbPromotion = promotionRepository.query(promotion.getId());
			Assert.notNull(dbPromotion, "该活动不存在！");
			if (!dbPromotion.getName().equals(promotion.getName())
					&& promotionRepository.existName(promotion.getName())) {
				return new GeneralResult(false, "活动名称已存在");
			}
			Long quota = promotion.getQuota();
			Long stock = dbPromotion.getStock();
			Long dataBaseStock = dbPromotion.getQuota();
			if (quota - dataBaseStock + stock < 0) {
				return new GeneralResult(false, "优惠卷数量不能小于已分配数量");
			}
			dbPromotion.setName(promotion.getName());
			dbPromotion.setDiscount(promotion.getDiscount());
			dbPromotion.setStartDate(promotion.getStartDate());
			dbPromotion.setEndDate(promotion.getEndDate());
			dbPromotion.setStatus(handleStatus(promotion.getStartDate(), promotion.getEndDate()).name());
			stock = stock + quota - dataBaseStock;
			dbPromotion.setQuota(quota);
			dbPromotion.setStock(stock);
			Boolean flag = promotionRepository.update(dbPromotion);
			if (!flag) {
				return new GeneralResult(false, "修改失败");
			}
			List<Coupon> coupons = couponRepository.listByPromotionId(dbPromotion.getId());
			for (Coupon coupon : coupons) {
				if (!coupon.getStatus().equals(Coupon.Status.used.name())) {
					Status status = handleStatus(dbPromotion.getStartDate(), dbPromotion.getEndDate());
					coupon.setStatus(status.equals(Status.expired) ? status.name() : Coupon.Status.unused.name());
				}
				coupon.setGmtStart(dbPromotion.getStartDate());
				coupon.setGmtFinish(dbPromotion.getEndDate());
				coupon.setEffectiveTime(dbPromotion.getStartDate());
				coupon.setExpiredTime(dbPromotion.getEndDate());
			}
			if (!ListTool.isEmpty(coupons)) {
				couponRepository.batchUpdate(coupons, coupons.size());
			}
			return new GeneralResult(true, dbPromotion, "修改成功");
		} catch (Exception e) {
			log.error("Get error message：{}", e);
			return new GeneralResult(false, "修改失败");
		}
	}

	@Override
	public GeneralResult detail(Long id, RequestContext context) {
		try {
			Promotion promotion = promotionRepository.query(id);
			if (!context.getCatalog().equals(RequestContext.Catalog.Manager)) {
				List<Coupon> coupons = couponRepository.listByPromotionId(promotion.getId(), context.getTenant());
				promotion.setStock(coupons.stream().filter(coupon -> coupon.getStatus().equals(Coupon.Status.unused.name())).count());
				promotion.setQuota((long) coupons.size());
			}
			return new GeneralResult(true, promotion, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：{}", e);
			return new GeneralResult(false, "查询失败");
		}
	}

	@Override
	public GeneralResult remove(Long id, RequestContext context) {
		String path = Promotion.class.getSimpleName() + "_" + id;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			Promotion promotion = promotionRepository.query(id);
			Assert.notNull(promotion, "数据不存在！");
			if (!promotion.getStatus().equals(Status.not_started.name())) {
				return new GeneralResult(false, "当前活动不允许删除");
			}
			Boolean flag = promotionRepository.remove(id);
			if (flag) {
				couponRepository.removeByPromotionId(id);
			}
			return flag ? new GeneralResult(true, "删除成功") : new GeneralResult(false, "删除失败");
		} catch (Exception e) {
			log.error("Get error message：{}", e);
			return new GeneralResult(false, "删除失败");
		}
	}


	@Override
	public GeneralResult allocation(Long id, Long tenantId, Integer count, RequestContext context) {
		String path = Promotion.class.getSimpleName() + "_" + id;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			Promotion database = promotionRepository.query(id);
			Assert.notNull(database, "该活动不存在！");
			if (database.getStock() - count < 0) {
				return new GeneralResult(false, "活动剩余优惠券不足");
			}
			List<Coupon> dbCoupons = couponRepository.listByPromotionId(id, tenantId);
			Integer max = dbCoupons.stream().map(Coupon::getNum).max(Comparator.comparing(x -> x)).orElse(0);
			List<Coupon> coupons = new ArrayList<>();
			for (int i = max; i < count + max; i++) {
				Coupon coupon = new Coupon();
				coupon.setSn(IdGenerator.getInstance().nextId());
				coupon.setNum(i+1);
				coupon.setName(coupon.getNum() + "_" + database.getName() + "_" + coupon.getSn());
				Status status = handleStatus(database.getStartDate(), database.getEndDate());
				coupon.setStatus(status.equals(Status.expired) ? status.name() : Coupon.Status.unused.name());
				coupon.setTenantId(tenantId);
				coupon.setGmtStart(database.getStartDate());
				coupon.setGmtFinish(database.getEndDate());
				coupon.setEffectiveTime(database.getStartDate());
				coupon.setExpiredTime(database.getEndDate());
				coupon.setPromotionId(id);
				coupons.add(coupon);
			}
			if (!ListTool.isEmpty(coupons)) {
				Boolean flag = couponRepository.batchSave(coupons, 1000);
				if (!flag) {
					return new GeneralResult(false, "优惠券分配失败");
				}
			}
			database.setStock(database.getStock() - count);
			Boolean flag = promotionRepository.update(database);
			if (!flag) {
				return new GeneralResult(false, "优惠券配额更新失败");
			}
			return new GeneralResult(true, "优惠券分配成功");
		} catch (Exception e) {
			log.error("Get error message：{}", e);
			return new GeneralResult(false, "优惠券分配失败");
		}
	}



	@Override
	public GeneralResult promotionTenants(Pager pager, Long id, RequestContext context) {
		try {
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			sorter.put("gmtCreate", Common.ONE);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
			List<Coupon> coupons = couponRepository.listByPromotionId(id);
			if (ListTool.isEmpty(coupons)) {
				return new GeneralResult(true, GridHelper.getBean(page, rows, 0, coupons), "查询成功");
			} else {
				List<Long> tenantIds = coupons.stream().map(Coupon::getTenantId).distinct().collect(Collectors.toList());
				params.add(new Param(MapTools.simpleMap("id", generateIds(tenantIds)), Sign.IN));
				GeneralResult result = tenantService.listTenants(new Pager(page, rows, params, null));
				if (result.isFailed()) {
					return result;
				}
				JSONObject dataObject = (JSONObject) result.getData();
				List<Tenant> javaList = dataObject.getJSONArray(Common.ROWS).toJavaList(Tenant.class);
				javaList.forEach(
						tenant -> {
							List<Coupon> tenantCoupons = coupons.stream().filter(coupon -> coupon.getTenantId().equals(tenant.getId())).collect(Collectors.toList());
							tenant.setCoupons(tenantCoupons.size());
							tenant.setAvailbleCoupon(Math.toIntExact(tenantCoupons.stream().filter(coupon -> coupon.getStatus().equals(Coupon.Status.unused.name())).count()));
							tenant.setUsedCoupon(Math.toIntExact(tenantCoupons.stream().filter(coupon -> coupon.getStatus().equals(Coupon.Status.used.name())).count()));
						});
				dataObject.put(Common.ROWS, javaList);
				result.setData(dataObject);
				return result;
			}
		} catch (Exception e) {
			log.error("Get error message：{}", e);
			return new GeneralResult(false, "查询租户失败：" + e);
		}
	}

	private String generateIds(List<Long> ids) {
		StringBuilder builder = new StringBuilder();
		for (int i = 0; i < ids.size(); i++) {
			builder.append(ids.get(i)).append(",");
		}
		builder.append(" 0 ");
		return builder.toString();
	}

	@Override
	public GeneralResult coupon(Pager pager, RequestContext context) {
		try {
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			sorter.put("num", "0");
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
			List<Coupon> list = couponRepository.list(page, rows, sorter, params);
			int count = couponRepository.count(params);
			GridBean gridBean = GridHelper.getBean(page, rows, count, list);
			return new GeneralResult(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：{}", e);
			return new GeneralResult(false, "查询失败");
		}
	}
}
