package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.context.ApplicationContext;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.entity.internel.*;
import com.bocloud.cos.enums.*;
import com.bocloud.cos.enums.ChargeCommon.ChargeMode;
import com.bocloud.cos.service.internal.FmsFlowService;
import com.bocloud.cos.service.internal.ImsInternalService;
import com.bocloud.cos.service.internal.SmsTenantService;
import com.bocloud.cos.intf.ResourceApplicationService;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.model.*;
import com.bocloud.cos.model.DurationModel.Mode;
import com.bocloud.cos.operate.*;
import com.bocloud.cos.params.CloneParams;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.utils.OrderItemUtils;
import com.google.common.collect.Lists;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.DateTools;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;
import java.util.Map.Entry;
import java.util.concurrent.TimeUnit;

/**
 * 服务订购
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月15日
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class ResourceApplicationServiceImpl implements ResourceApplicationService {

    private final ServiceOrderRepository serviceOrderRepository;
    private final AmqpTemplate amqpTemplate;
    private final OrderItemRepository orderItemRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceCategoryRepository serviceCategoryRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final ServiceSlaRepository serviceSlaRepository;
    private final SlaInstanceRepository slaInstanceRepository;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final SmsTenantService tenantService;
    private final ResourceQuotaService resourceQuotaService;
    private final ImsInternalService imsInternalService;
    private final LockFactory lockFactory;
    private final TenantAccountRepository accountRepository;
    private final ServiceFlowConfigRepository serviceFlowConfigRepository;
    private final FmsFlowService fmsFlowService;
    private final ChargeItemRepository chargeItemRepository;
    private final ResourceEventPublisher resourceEventPublisher;
    private final CouponRepository couponRepository;

    @Override
    @Transactional
    public GeneralResult<String> process(ApplicationParam applicationParam, RequestContext context) {
        log.info("接收到订购消息：{}", JSONObject.toJSONString(applicationParam));
        // 申请校验规格 校验服务是否存在等
        ApplicationContext applicationContext = buildApplicationContext(applicationParam, context);
        // 生成订单
        ServiceOrder serviceOrder = this.createOrder(applicationContext, context);
        // 预生成生成订单项，计算配额并校验
        List<OrderItem> orderItems = applicationContext.getOperate().buildApplyItem(applicationContext.getService().getCode(),
                applicationParam.getEmption(), applicationParam.getElements());
        serviceOrder.setItems(orderItems);
        applicationContext.setServiceOrder(serviceOrder);

        // 校验配额信息
        this.validateQuota(applicationContext);

        // 保存订单项
        this.saveOrderItems(applicationContext);

        // 保存SLA信息
        if (null != applicationContext.getSla()) {
            SlaInstance slaInstance = new SlaInstance(applicationContext.getSla().getName(), applicationContext.getSla().getContent(),
                    new Date(), context.getTarget(), applicationContext.getSla().getServiceId(), applicationContext.getSlaId(),
                    applicationContext.getServiceOrder().getId());
            slaInstanceRepository.save(slaInstance);
        }
        // 封装资源申请的必要参数到 applicationParam.configs
        this.warpApplicationConfig(applicationContext);

        applicationParam.setContext(context);
        Long flowId = serviceFlowConfigRepository.getServiceFlowId(applicationContext.getService().getId(), ApplyCategory.ApplicationOperate);

        Boolean noFlow = Optional.ofNullable(applicationContext.getParams().getConfigs().getBoolean("noFlow")).orElse(false);
        if (null != flowId && !noFlow) {
            // 订单、订单项状态变更为审批中
            orderItems.forEach(item -> {
                item.setStatus(OrderItemStatus.Audit);
                orderItemRepository.update(item);
            });
            applicationContext.getServiceOrder().setOrderStatus(ChargeCommon.OrderStatus.audit);
            serviceOrderRepository.update(applicationContext.getServiceOrder());
            applicationParam.setOrder(applicationContext.getServiceOrder());
            Map<String, Object> content = getParams(ApplyCategory.ApplicationOperate.name(), JSONObject.toJSONString(applicationParam),
                    "申请审核服务【" + applicationContext.getService().getName() + "】使用", context);
            content.put("order", JSONObject.toJSONString(applicationContext.getServiceOrder()));
            content.put("requestContext", JSONObject.toJSONString(context));
            GeneralResult<?> execution = fmsFlowService.startInstanceById(flowId, context, applicationParam.getLocation().getProjectId(), content);
            if (execution.isFailed()) {
                throw new RuntimeException("服务申请使用审核流程启动失败");
            }
            return new GeneralResult<>(true, "服务申请使用审核流程启动成功");
        } else {
            // 扣除配额
            if (null != applicationContext.getResourceQuotas() && !applicationContext.getResourceQuotas().isEmpty()) {
                resourceQuotaService.quotaAction(applicationContext.getResourceQuotas(), ResourceQuota.Action.Order,
                        applicationContext.getTenant().getId());
            }
            // 下发到底层服务的时候按年计费时间做特殊处理
            if (Mode.YEAR == applicationContext.getParams().getEmption().getDuration().getMode()) {
                EmptionModel paramEmption = applicationParam.getEmption();
                DurationModel duration = paramEmption.getDuration();
                duration.setAmount(duration.getAmount() * 12);
                duration.setMode(Mode.MONTH);
                paramEmption.setDuration(duration);
                applicationParam.setEmption(paramEmption);
            }
            applicationContext.getServiceOrder().setApplyParam(JSONObject.toJSONString(applicationParam));
            serviceOrderRepository.update(applicationContext.getServiceOrder());
            // 预付费，进入到待支付状态
            if (ChargeMode.PreCharge == applicationContext.getServiceOrder().getChargeMode()) {
                applicationContext.getServiceOrder().setOrderStatus(ChargeCommon.OrderStatus.unpay);
                serviceOrderRepository.update(applicationContext.getServiceOrder());
                return new GeneralResult<>(true, applicationContext.getServiceOrder().getSn(), "申请已提交,支付订单后资源即可创建");
            }
            // 后付费
            applicationContext.getServiceOrder().setOrderStatus(ChargeCommon.OrderStatus.paid);
            serviceOrderRepository.update(applicationContext.getServiceOrder());
            applicationParam.setOrder(applicationContext.getServiceOrder());
            if ("vmware.nsx.vpc".equalsIgnoreCase(applicationContext.getService().getCode())) {
                log.info("后付费NSX-T Vpc申请参数结构：" + JSONObject.toJSONString(applicationParam));
                amqpTemplate.convertAndSend("network.creation.queue", applicationParam);
            } else {
                List<ApplicationParam> messageParams = Lists.newArrayList(applicationParam);
                log.info("后付费资源申请参数结构：" + JSONObject.toJSONString(messageParams));
                if (applicationContext.getOperate() instanceof PaasResourceOperate) {
                    amqpTemplate.convertAndSend("paas.resource.creation.queue", messageParams);
                } else if (applicationContext.getOperate() instanceof IaasResourceOperate) {
                    amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                } else {
                    amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                }
            }
            return new GeneralResult<>(true, "申请已提交,请等待资源创建。");
        }
    }

    @Override
    @Transactional
    public GeneralResult<RosApplicationParam> process(RosApplicationParam rosApplicationParam) {
        log.info("接收到资源编排订购请求：{}", JSONObject.toJSONString(rosApplicationParam));
        RequestContext context = rosApplicationParam.getContext();
        CloudService service = cloudServiceRepository.queryByCode(rosApplicationParam.getService());
        Assert.notNull(service, "该服务不存在");
        Assert.isTrue(CloudService.Status.PUBLISHED.name().equals(service.getStatus()), "服务未发布");
        Tenant tenant = tenantService.detail(context.getTenant());
        Assert.notNull(tenant, "申请人所在租户不存在");
        // 生成订单
        ServiceOrder serviceOrder = new ServiceOrder();
        serviceOrder.setServiceCode(rosApplicationParam.getService());
        // 生成订单号
        String orderSn = this.generateSn();
        serviceOrder.setSn(orderSn);
        ChargeMode chargeMode = ChargeCommon.ChargeMode.PostCharge;
        serviceOrder.setChargeMode(chargeMode);
        serviceOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
        serviceOrder.setType(ChargeCommon.Type.apply);
        serviceOrder.setAutoRenew(false);
        if (Catalog.User == context.getCatalog()) {
            serviceOrder.setUserId(context.getTarget());
        }
        serviceOrder.setTenantId(context.getTenant());
        serviceOrder.setResourceStatus(ChargeCommon.ResourceStatus.creating);
        serviceOrderRepository.save(serviceOrder);

        List<OrderItem> allOrderItems = new ArrayList<>();
        List<ResourceQuota> resourceQuotas = new ArrayList<>();
        for (ApplicationParam applicationParam : rosApplicationParam.getNodeParams()) {
            // 申请校验规格 校验服务是否存在等
            ApplicationContext applicationContext = buildApplicationContext(applicationParam, context);
            // 预生成生成订单项，计算配额并校验
            List<OrderItem> orderItems = applicationContext.getOperate().buildApplyItem(applicationContext.getService().getCode(),
                    applicationParam.getEmption(), applicationParam.getElements());
            orderItems.forEach(orderItem -> orderItem.setNodeId(applicationParam.getNodeId()));
            serviceOrder.setItems(orderItems);
            allOrderItems.addAll(orderItems);
            applicationContext.setServiceOrder(serviceOrder);
            // 校验配额信息
            this.validateQuota(applicationContext);
            // 保存订单项
            this.saveOrderItems(applicationContext);
            // 保存SLA信息
            if (null != applicationContext.getSla()) {
                SlaInstance slaInstance = new SlaInstance(applicationContext.getSla().getName(), applicationContext.getSla().getContent(),
                        new Date(), context.getTarget(), applicationContext.getSla().getServiceId(), applicationContext.getSlaId(),
                        applicationContext.getServiceOrder().getId());
                slaInstanceRepository.save(slaInstance);
            }
            // 封装资源申请的必要参数到 applicationParam.configs
            this.warpApplicationConfig(applicationContext);
            if (null != applicationContext.getResourceQuotas() && !applicationContext.getResourceQuotas().isEmpty()) {
                resourceQuotas.addAll(applicationContext.getResourceQuotas());
            }
        }
        serviceOrder.setItems(allOrderItems);
        // 扣除配额
        if (!resourceQuotas.isEmpty()) {
            resourceQuotaService.quotaAction(resourceQuotas, ResourceQuota.Action.Order, context.getTenant());
        }
        serviceOrder.setApplyParam(JSONObject.toJSONString(rosApplicationParam));
        serviceOrderRepository.update(serviceOrder);
        // 后付费
        serviceOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
        serviceOrderRepository.update(serviceOrder);
        rosApplicationParam.setOrder(serviceOrder);
        return new GeneralResult<>(true, rosApplicationParam, "申请已提交,请等待资源创建。");
    }

    @Transactional
    @Override
    public void rosCallBack(boolean success, long orderId, String message) {
        ServiceOrder order = serviceOrderRepository.query(orderId);
        if (null == order) {
            log.error("Service order does not exists!");
            return;
        }
        CloudService cloudService = cloudServiceRepository.queryByCode(order.getServiceCode());
        // 申请用户
        RequestContext context;
        if (null == order.getUserId()) {
            context = new RequestContext(RequestContext.Catalog.Tenant, order.getTenantId(), order.getTenantId(), null, null);
        } else {
            context = new RequestContext(RequestContext.Catalog.Tenant, order.getUserId(), order.getTenantId(), null, null);
        }
        // 修改订单状态
        order.setResourceStatus(success ? ChargeCommon.ResourceStatus.ready : ChargeCommon.ResourceStatus.failed);
        serviceOrderRepository.update(order);

        List<OrderItem> orderItems = orderItemRepository.listByOrderId(orderId);
        orderItems.forEach(orderItem -> {
            orderItem.setErrorMsg(message);
            orderItem.setStatus(success ? OrderItemStatus.OpenedSuccess : OrderItemStatus.OpenedFailure);
            //更新订单项
            orderItem.setEffectiveTime(new Date());
            orderItem.setExpiredTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
            orderItemRepository.update(orderItem);
            if (success) {
                if (ChargeCommon.ChargeMode.PostCharge.equals(order.getChargeMode())) {
                    // 非公有云 后付费添加到跑批计费内容中
                    CloudService itemCloudService = cloudServiceRepository.queryByCode(orderItem.getServiceCode());
                    if (!itemCloudService.getIsPublic()) {
                        ChargeItem chargeItem = new ChargeItem();
                        chargeItem.setServiceCode(orderItem.getServiceCode());
                        chargeItem.setPrice(orderItem.getFinalPrice());
                        chargeItem.setGmtCreate(new Date());
                        chargeItem.setItemSn(orderItem.getSn());
                        chargeItem.setItemId(orderItem.getId());
                        chargeItem.setDeleted(false);
                        chargeItem.setType(ChargeItem.Type.loop);
                        chargeItemRepository.baseSave(chargeItem);
                    }
                }
            } else {
                // 如果开通失败 回滚配额
                List<QuotaBean> quotas = OrderItemUtils.getItemQuotaBeans(Lists.newArrayList(orderItem));
                log.info("open failed call back quota,the quota is {}", JSONObject.toJSONString(quotas));
                resourceQuotaService.callBackQuota(quotas, order.getTenantId());
            }
            OperateResult operateResult = new OperateResult(success, message, "resource.ros", null,
                    OperateResult.OperateCategory.ResourceEvent, context);
            resourceEventPublisher.send(operateResult);
        });
        // 发送站内信
        String msgContent = "尊敬的客户，您好！您的" + cloudService.getName() + "于" + DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss")
                + (success ? "订购成功" : "订购失败") + "(订单编号: " + order.getSn() + ")，请登录控制台查看及管理。";
        if (!success) {
            msgContent += " 报错信息：\n " + message;
        }
        NotifyMessage notifyMessage = new NotifyMessage(context, cloudService.getName(), msgContent, true, false);
        amqpTemplate.convertAndSend("event.notice.queue", notifyMessage);
    }

    /**
     * 生成订单
     *
     * @param applicationContext 服务订购上下文
     * @param context            用户
     * @return 订单
     */
    private ServiceOrder createOrder(ApplicationContext applicationContext, RequestContext context) {
        ServiceOrder order = new ServiceOrder();
        order.setServiceCode(applicationContext.getService().getCode());
        // 生成订单号
        String orderSn = this.generateSn();
        order.setSn(orderSn);
        ChargeMode chargeMode = ChargeCommon.ChargeMode.PostCharge;
        if (Mode.HOUR != applicationContext.getParams().getEmption().getDuration().getMode()) {
            chargeMode = ChargeCommon.ChargeMode.PreCharge;
        }
        order.setChargeMode(chargeMode);
        order.setSlaAgreed(null != applicationContext.getSlaId() && 0L != applicationContext.getSlaId());
        if (ChargeMode.PostCharge == chargeMode) {
            order.setOrderStatus(ChargeCommon.OrderStatus.paid);
        } else {
            order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
        }
        order.setType(ChargeCommon.Type.apply);
        order.setGmtCreate(new Date());
        order.setAutoRenew(false);
        // 设置订单归属
        order.setVendorId(applicationContext.getParams().getLocation().getVendorId());
        order.setBusinessId(applicationContext.getParams().getLocation().getBusinessId());
        order.setProjectId(applicationContext.getParams().getLocation().getProjectId());
        order.setUserId(applicationContext.getParams().getLocation().getOwnerId());
        order.setTenantId(context.getTenant());
        if (context.getCatalog().equals(Catalog.User) && null == order.getUserId()) {
            order.setUserId(context.getTarget());
        }
        order.setResourceStatus(ChargeCommon.ResourceStatus.creating);
        serviceOrderRepository.save(order);
        return order;
    }

    /**
     * 持久化保存订单项
     *
     * @param applicationContext 服务订购上下文
     */
    private void saveOrderItems(ApplicationContext applicationContext) {
        if (ListTool.isEmpty(applicationContext.getServiceOrder().getItems())) {
            return;
        }
        // 处理优惠券
        processCoupon(applicationContext, applicationContext.getServiceOrder().getSn());
        // 保存订单项
        for (OrderItem item : applicationContext.getServiceOrder().getItems()) {
            item.setOrderId(applicationContext.getServiceOrder().getId());
            List<ElementModel> elementModels = JSONArray.parseArray(item.getElements(), ElementModel.class);
            if (elementModels.isEmpty()) {
                ElementModel elementModel = new ElementModel();
                elementModel.setRatio(applicationContext.getTenant().getDiscount());
                elementModels = Lists.newArrayList(elementModel);
            } else {
                elementModels.forEach(elementModel -> elementModel.setRatio(applicationContext.getTenant().getDiscount()));
            }
            item.setElements(JSONObject.toJSONString(elementModels));
            item.setFinalPrice(item.getBasicPrice().multiply(applicationContext.getTenant().getDiscount()));
            orderItemRepository.save(item);
        }
    }

    /**
     * 处理优惠券
     *
     * @param applicationContext
     * @param orderSn
     */
    private void processCoupon(ApplicationContext applicationContext, String orderSn) {
        try {
            if (null != applicationContext.getParams().getCouponId()) {
                couponRepository.updateStatus(applicationContext.getParams().getCouponId(), Coupon.Status.used.name(), orderSn);
            }
        } catch (Exception e) {
            log.error("处理优惠券异常：{}", e);
        }
    }

    /**
     * 生成订单号
     *
     * @return 订单号
     */
    private String generateSn() {
        String path = IdGenerator.class.getSimpleName() + "_orderSn";
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
            return IdGenerator.getInstance().nextId();
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage(), e);
        }
    }

    /**
     * 创建服务申请上下文
     *
     * @param applicationParam 订购参数
     * @param context          当前用户
     * @return 服务订购上下文
     */
    private ApplicationContext buildApplicationContext(ApplicationParam applicationParam, RequestContext context) {
        ApplicationContext applicationContext = new ApplicationContext();
        applicationContext.setParams(applicationParam);
        log.info("context:" + JSON.toJSONString(context));
        Tenant tenant = tenantService.detail(context.getTenant());
        if (null != tenant) {
            applicationContext.setTenant(tenant);
        } else {
            log.error("查询租户详情失败");
        }
        this.validateParams(applicationParam, applicationContext, context);
        // 资源操作实例
        ResourceOperate applyOperate = ResourceOperateFactory.getInstance(applicationContext.getService().getCode());
        Assert.notNull(applyOperate, "暂不支持该类服务订购操作");
        applicationContext.setOperate(applyOperate);
        // 资源类型，用于纳管创建资源时的类型标识
        ResourceCategory resourceCategory = ServiceResourceMapping.getResourceCategory(applicationContext.getService().getCode());
        applicationContext.setResourceCategory(resourceCategory);
        return applicationContext;
    }

    /**
     * 校验服务申请参数，构造服务订购上下文
     *
     * @param param              订购参数
     * @param applicationContext 服务订购上下文
     * @param context            当前用户
     */
    private void validateParams(ApplicationParam param, ApplicationContext applicationContext, RequestContext context) {
        CloudService service = cloudServiceRepository.queryByCode(param.getService());
        log.debug("service:" + param.getService());
        log.debug("CloudService:" + JSON.toJSONString(service));
        Assert.notNull(service, "该服务不存在");
        Assert.isTrue("PUBLISHED".equals(service.getStatus()), "服务未发布");
        Tenant tenant = tenantService.detail(context.getTenant());
        Assert.notNull(tenant, "申请人所在租户不存在");
        // 商品sku
        List<ElementModel> elements = param.getElements();
        // 服务协议信息
        Long slaId = param.getSla();
        ServiceSla sla = null;
        if (null == slaId) {
            List<ServiceSla> slas = serviceSlaRepository.listByService(service.getId(), ServiceSla.Status.enable.name());
            if (!ListTool.isEmpty(slas)) {
                sla = slas.get(0);
                slaId = sla.getId();
            }
        } else {
            sla = serviceSlaRepository.query(slaId);
            Assert.notNull(sla, "该服务的SLA协议不存在，无法订购");
        }
        Long skuId = null;
        for (ElementModel element : elements) {
            if (service.getCode().equals(element.getServiceCode())) {
                skuId = element.getSkuId();
                break;
            }
        }
        ServiceCategory category = null;
        ServiceSku sku = null;
        if (null != skuId && 0L != skuId) {
            sku = serviceSkuRepository.query(skuId);
            Assert.notNull(sku, "该服务产品不存在");
            category = serviceCategoryRepository.query(sku.getCategoryId());
            Assert.notNull(category, "该服务类型不存在");
            // sku不为空，则说明需要走计费，需要校验租户授信额度
            TenantAccount account = accountRepository.getAccount(context.getTenant());
            Assert.notNull(account, "未查询到租户授信额度");
        }
        applicationContext.setService(service);
        applicationContext.setSlaId(slaId);
        applicationContext.setSla(sla);
        applicationContext.setSkuId(skuId);
        applicationContext.setSku(sku);
        applicationContext.setCategory(category);

        // 校验优惠券是否已过期
        if (null != applicationContext.getParams().getCouponId()) {
            Coupon coupon = couponRepository.getCouponById(applicationContext.getParams().getCouponId());
            if (null != coupon && null != coupon.getDiscount()) {
                long expireTime = coupon.getExpiredTime().getTime();
                long currentTime = System.currentTimeMillis();
                Assert.isTrue(!(currentTime - expireTime > 0), "所用优惠券已过期");
                BigDecimal tenantDiscount = tenant.getDiscount().compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ONE : tenant.getDiscount();
                applicationContext.getTenant().setDiscount(tenantDiscount.multiply(coupon.getDiscount().multiply(BigDecimal.valueOf(0.1))));
            }
        }
    }

    /**
     * 校验配额信息
     *
     * @param applicationContext 订购申请上下文
     */
    private void validateQuota(ApplicationContext applicationContext) {
        // 计算订单项配额
        Map<String, Integer> quotaMap = OrderItemUtils.getItemQuota(applicationContext.getServiceOrder().getItems());

        if (!applicationContext.getTenant().getQuotaAble()) {
            // 租户未开启配额,不需要计算配额
            return;
        }
        if (quotaMap.isEmpty()) {
            return;
        }
        List<ResourceQuota> list = resourceQuotaRepository.listByTenantId(applicationContext.getTenant().getId());
        Assert.notEmpty(list, "未查询到配额信息");
        Map<String, ResourceQuota> map = new HashMap<>(8);
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (Entry<String, Integer> entry : quotaMap.entrySet()) {
            String code = entry.getKey();
            String quotaName;
            if (QuotaCategory.CPU.getCode().equals(code)) {
                quotaName = QuotaCategory.CPU.getName();
            } else if (QuotaCategory.MEM.getCode().equals(code)) {
                quotaName = QuotaCategory.MEM.getName();
            } else if (QuotaCategory.DISK.getCode().equals(code)) {
                quotaName = QuotaCategory.DISK.getName();
            } else {
                CloudService cloudService = cloudServiceRepository.queryByCode(code);
                if (StringUtils.isBlank(cloudService.getUnit())) {
                    log.debug(cloudService.getName() + "服务未开启配额");
                    continue;
                }
                quotaName = cloudService.getName();
            }
            if (!map.containsKey(entry.getKey())) {
                throw new RuntimeException("未查询到" + quotaName + "配额信息");
            }
            ResourceQuota quota = map.get(entry.getKey());
            if (entry.getValue() + quota.getUsed() > quota.getQuota()) {
                throw new RuntimeException("剩余" + quotaName + "配额不足");
            }
            quota.setQuotaActionValue(entry.getValue().longValue());
            quotas.add(quota);
        }
        applicationContext.setResourceQuotas(quotas);
    }

    /**
     * 封装资源申请的必要参数
     *
     * @param applicationContext 服务申请上下文
     */
    private void warpApplicationConfig(ApplicationContext applicationContext) {
        JSONObject configs = applicationContext.getParams().getConfigs();
        if (null == configs) {
            configs = new JSONObject();
        }
        configs.put("skuId", applicationContext.getSkuId());
        if (null != applicationContext.getSku()) {
            configs.put("skuName", applicationContext.getSku().getName());
        }
        if (null != applicationContext.getCategory()) {
            configs.put("categoryId", applicationContext.getCategory().getId());
            configs.put("categoryCode", applicationContext.getCategory().getCode());
            configs.put("categoryProps", applicationContext.getCategory().getProps());
        }
        // 资源类型，用于纳管创建资源时的类型标识
        configs.put("category", applicationContext.getResourceCategory().name());
        if (StringUtils.isNotEmpty(applicationContext.getService().getTemplateName())) {
            configs.put("templateName", applicationContext.getService().getTemplateName());
        }
        applicationContext.getParams().setConfigs(configs);
    }

    @Override
    @Transactional
    public void clone(CloneParams cloneParams, RequestContext context) {
        // 生成克隆参数
        log.info("接收到克隆消息：{}", JSONObject.toJSON(cloneParams));
        Assert.notNull(cloneParams, "参数格式错误");
        Long templateId = cloneParams.getTemplateId();
        String instance = cloneParams.getInstance();
        JSONObject config = new JSONObject();
        config.put("templateId", templateId);
        config.put("instance", instance);
        Assert.notNull(instance, "未找到资源关联的子订单");
        OrderItem item = orderItemRepository.queryBySn(instance);
        Assert.notNull(item, "未找到资源关联的子订单");
        ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
        Assert.notNull(order, "未找到资源关联的订单");
        GeneralResult<?> serverResult = imsInternalService.serverDetail(templateId);
        Assert.isTrue(serverResult.isSuccess(), serverResult.getMessage());
        CloudServer server = JSONObject.parseObject(JSONObject.toJSONString(serverResult.getData()), CloudServer.class);
        // 查询磁盘列表
        List<Volume> volumes;
        if (item.getServiceCode().contains("smartx.")) {
            volumes = imsInternalService.getVolumesForSmartX(server.getId(), server.getVendorId(), server.getVendorType());
        } else {
            volumes = imsInternalService.listByVmId(server.getId(), server.getVendorType());
        }
        // 系统盘
        Volume sysDisk = volumes.stream()
                .filter(volume -> "Hard disk 1".equalsIgnoreCase(volume.getName()) || "硬盘 1".equalsIgnoreCase(volume.getName()))
                .findFirst().orElse(null);
        Assert.notNull(sysDisk, "未查询到系统盘信息");
        Long skuId = sysDisk.getSkuId();
        if (null != skuId && 0L != skuId) {
            ServiceSku diskSku = serviceSkuRepository.query(skuId);
            Assert.notNull(diskSku, "云主机系统盘关联的产品SKU未找到");
            config.put("diskCategoryId", diskSku.getCategoryId());
        }
        List<NetworkCard> netWordCards = imsInternalService.listNetworkCards(server.getId(), server.getVendorType());
        JSONArray networkCardConfigs = new JSONArray();
        // 原云主机配置
        if ("Manual".equalsIgnoreCase(cloneParams.getIpPolicy())) {
            Assert.isTrue(netWordCards.size() == cloneParams.getAddresses().size(), "请分配IP,并确认分配IP数与云主机网卡数一致");
        }
        for (int i = 0; i < netWordCards.size(); i++) {
            JSONObject networkCardConfig = new JSONObject();
            networkCardConfig.put("ipPoolId", cloneParams.getIpPoolId());
            networkCardConfig.put("ipPolicy", cloneParams.getIpPolicy());
            networkCardConfig.put("networkCardId", netWordCards.get(i).getId());
            networkCardConfig.put("portGroupId", cloneParams.getPortGroupId());
            networkCardConfig.put("address", "Manual".equalsIgnoreCase(cloneParams.getIpPolicy()) ? Lists.newArrayList(cloneParams.getAddresses().get(i)) : cloneParams.getAddresses());
            networkCardConfigs.add(networkCardConfig);
        }
        ApplicationParam newApplication = new ApplicationParam();
        LocationModel locationModel = new LocationModel();
        locationModel.setVendorId(server.getVendorId());
        PoolGroup poolGroup = imsInternalService.getPoolGroup(server.getPoolGroupId());
        Assert.notNull(poolGroup, "没找相应的资源池");
        locationModel.setAz(poolGroup.getAvailablitiyZone());
        locationModel.setRegion(poolGroup.getRegion());
        locationModel.setPoolGroupId(poolGroup.getId());
        locationModel.setProjectId(server.getProjectId());
        locationModel.setBusinessId(server.getBusinessId());
        locationModel.setCreatorId(server.getCreatorId());
        locationModel.setMenderId(server.getMenderId());
        locationModel.setOwnerId(server.getOwnerId());
        locationModel.setRegion(server.getRegion());
        locationModel.setVendorId(server.getVendorId());
        locationModel.setTenantId(server.getTenantId());
        locationModel.setVendorType(server.getVendorType());
        newApplication.setLocation(locationModel);
        DurationModel duration = new DurationModel();
        duration.setMode(Mode.HOUR);
        EmptionModel emption = new EmptionModel();
        emption.setCount(1);
        emption.setAutoRenew(false);
        emption.setDuration(duration);
        newApplication.setEmption(emption);
        newApplication.setService(item.getServiceCode());

        List<ElementModel> elementModels = new ArrayList<>();
        Integer cpu = server.getCpu();
        BigDecimal memory = server.getMemory();
        // cpu内存规格
        ElementModel element = new ElementModel();
        element.setServiceCode(newApplication.getService());
        Long skuId1 = Optional.ofNullable(server.getSkuId()).orElse(0L);
        element.setSkuId(skuId1);
        element.setCategoryId(server.getCategoryId());
        SpecModel specModel = new SpecModel();
        specModel.setCpu(cpu);
        specModel.setMemory(memory);
        element.setSpecs(Lists.newArrayList(specModel));
        elementModels.add(element);
        // 磁盘规格
        for (Volume volume : volumes) {
            ElementModel volumeElement = new ElementModel();
            Long volumeSkuId = volume.getSkuId();
            if (null != volumeSkuId && 0L != volumeSkuId) {
                ServiceSku diskSku = serviceSkuRepository.query(volumeSkuId);
                volumeElement.setSkuId(volumeSkuId);
                volumeElement.setCategoryId(diskSku.getCategoryId());
            }
            volumeElement.setServiceCode("vmware.storage.disk");
            SpecModel diskSpecModel = new SpecModel();
            diskSpecModel.setDisk(volume.getSize());
            volumeElement.setSpecs(Lists.newArrayList(diskSpecModel));
            elementModels.add(volumeElement);
        }
        newApplication.setElements(elementModels);
        config.put("name", cloneParams.getName());
        config.put("isAddShterm", cloneParams.getIsAddShterm());
        config.put("cpu", cpu);
        config.put("memory", memory);
        config.put("disk", sysDisk.getSize());
        config.put("skuId", item.getSkuId());
        config.put("category", ResourceCategory.Computer.name());
        config.put("networkCardConfigs", networkCardConfigs);
        if (StringUtils.isNotEmpty(server.getPswd())) {
            Encryptor encryptor = new AESEncryptor();
            config.put("password", encryptor.encrypt(server.getPswd(), null));
        }
        newApplication.setConfigs(config);
        newApplication.setContext(context);
        // 申请校验规格 校验服务是否存在等
        ApplicationContext applicationContext = buildApplicationContext(newApplication, context);

        // 生成订单
        ServiceOrder cloneOrder = this.createOrder(applicationContext, context);

        // 预生成生成订单项，计算配额并校验
        List<OrderItem> orderItems = applicationContext.getOperate().buildApplyItem(applicationContext.getService().getCode(),
                newApplication.getEmption(), newApplication.getElements());
        cloneOrder.setItems(orderItems);
        applicationContext.setServiceOrder(cloneOrder);
        // 校验配额信息
        this.validateQuota(applicationContext);

        // 扣除配额
        if (null != applicationContext.getResourceQuotas() && !applicationContext.getResourceQuotas().isEmpty()) {
            resourceQuotaService.quotaAction(applicationContext.getResourceQuotas(), ResourceQuota.Action.Order,
                    applicationContext.getTenant().getId());
        }

        // 保存订单项
        this.saveOrderItems(applicationContext);

        cloneOrder.setApplyParam(JSONObject.toJSONString(newApplication));
        newApplication.setOrder(cloneOrder);
        log.info("克隆消息{},发送成功", JSONObject.toJSONString(Lists.newArrayList(newApplication)));
        amqpTemplate.convertAndSend("iaas.resource.creation.queue", Lists.newArrayList(newApplication));
    }

    private Map<String, Object> getParams(String category, String params, String theme, RequestContext context) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("ApplyCategory", category);
        map.put("ApplyParams", params);
        map.put("ApplyTheme", theme);
        map.put("ApplyContext", context);
        return map;
    }

}
