package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.QuotaHistory;
import com.bocloud.cos.entity.ResourceQuota;
import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.cos.enums.QuotaCategory;
import com.bocloud.cos.service.internal.SmsTenantService;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.model.QuotaBean;
import com.bocloud.cos.model.QuotaModel;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.QuotaHistoryRepository;
import com.bocloud.cos.repository.ResourceQuotaRepository;
import com.bocloud.cos.repository.ServiceCatalogRepository;
import com.bocloud.cos.entity.internel.Tenant;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.License;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.model.Result;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.license.LicenseDaemon;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.groupingBy;
import static java.util.stream.Collectors.toMap;

/**
 * 资源配额操作service实现类
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年11月23日
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class ResourceQuotaServiceImpl implements ResourceQuotaService {

    private final ResourceQuotaRepository quotaRepository;
    private final LockFactory lockFactory;
    private final SmsTenantService smsTenantService;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceCatalogRepository serviceCatalogRepository;
    private final QuotaHistoryRepository quotaHistoryRepository;
    private final SmsTenantService tenantService;
    private final CurrentService currentService;

    /**
     * 直接删除配额 (cos删除租户或项目调用)父租户配额更新及记录调用配额更新方法已处理
     *
     * @param tenantId
     * @param requestContext
     * @return
     */
    @Override
    public Result delete(Long tenantId, RequestContext requestContext) {
        try {
            List<ResourceQuota> quotas = quotaRepository.listByTenantId(tenantId);
            ArrayList<QuotaHistory> histories = new ArrayList<>();
            for (ResourceQuota resourceQuota : quotas) {
                if (resourceQuota.getQuota() != null && resourceQuota.getQuota() > 0) {
                    QuotaHistory quotaHistory =
                            new QuotaHistory(tenantId, resourceQuota.getQuota(), resourceQuota.getCode(),
                                    resourceQuota.getUnit(), resourceQuota.getCategory(), ResourceQuota.Action.Decrease,
                                    new Date(), requestContext.getTarget());
                    histories.add(quotaHistory);
                }
            }
            quotaRepository.delete(tenantId);
            if (!ListTool.isEmpty(histories)) {
                histories.forEach(quotaHistoryRepository::save);
            }
            return new Result(true, "删除成功");
        } catch (Exception e) {
            log.error("Get error message:", e);
            return new Result(false, "删除失败");
        }
    }

    /**
     * 删除服务配额
     *
     * @param target 目标id
     * @param code   配额编码
     * @return 结果
     */
    @Override
    @Transactional
    public Result delete(Long target, String category, String code, RequestContext requestContext) {
        ArrayList<QuotaHistory> histories = new ArrayList<>();
        List<ResourceQuota> targetQuotas = quotaRepository.list(target, category, code);
        // 自身配额记录删除量
        for (ResourceQuota quota : targetQuotas) {
            if (quota.getQuota() != null && quota.getQuota() > 0) {
                QuotaHistory history = new QuotaHistory(quota.getTarget(), quota.getQuota(), quota.getCode(), quota.getUnit(),
                        quota.getCategory(), ResourceQuota.Action.Decrease, new Date(), requestContext.getTarget());
                histories.add(history);
            }
        }
        // 1.删除配额 2.更新父租户配额 3.更新配额记录
        quotaRepository.delete(target, category, code);
        if (!ListTool.isEmpty(histories)) {
            histories.forEach(quotaHistoryRepository::save);
        }
        return new GeneralResult<>(true, "删除配额成功");
    }

    @Override
    public GeneralResult getQuota(String condition, String code, Long target) {
        try {
            if ("getQuotaTop5".equals(condition)) {
                List<ResourceQuota> top5Quotas = quotaRepository.quotaTop5(code);
                return new GeneralResult<>(true, top5Quotas, "查询成功");
            }
            ResourceQuota resourceQuota = quotaRepository.query(target, code);
            if (null == resourceQuota) {
                return new GeneralResult<>(false, "数据不存在");
            }
            return new GeneralResult<>(true, resourceQuota, "查询成功");
        } catch (Exception e) {
            log.error("Get error message:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result assign(Long tenantId, List<ResourceQuota> quotas, RequestContext requestContext) {
        // 配额目标类别
        String path = Tenant.class.getSimpleName() + "_quota_" + tenantId;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Tenant tenant = smsTenantService.detail(tenantId);
            Assert.notNull(tenant, "租户信息不存在");
            // 判断该租户是否进行过配额操作
            List<ResourceQuota> oldQuotas = new ArrayList<>();
            for (ResourceQuota quota : quotas) {
                String quotaName = "";
                CloudService cloudService = cloudServiceRepository.queryByCode(quota.getCode());
                if (cloudService != null) {
                    quotaName = cloudService.getName();
                } else {
                    for (QuotaCategory quotaCategory : QuotaCategory.values()) {
                        if (quotaCategory.getCode().equals(quota.getCode())) {
                            quotaName = quotaCategory.getName();
                        }
                    }
                }
                quota.setName(quotaName);
                ResourceQuota oldQuota = quotaRepository.query(tenantId, quota.getCode());
                if (oldQuota != null) {
                    oldQuotas.add(oldQuota);
                    quota.setOldQuota(oldQuota.getQuota());
                }
            }
            //汇总子租户和项目配额总和，校验新配额是否通过
            Result checkResult = this.ownerCheck(oldQuotas, quotas);
            if (checkResult.isFailed()) {
                return checkResult;
            }
            // 保存或更新配额
            Result result = this.process(oldQuotas, quotas, requestContext);
            if (result.isFailed()) {
                return new Result(false, "租户配额失败");
            }
            return new Result(true, "配额成功");
        } catch (Exception e) {
            log.error("Get error message:", e);
            return new Result(false, "配额失败");
        }
    }

    /**
     * 校验自身配额是否合法
     *
     * @param oldQuotas
     * @param newQuotas
     * @return
     */
    private Result ownerCheck(List<ResourceQuota> oldQuotas, List<ResourceQuota> newQuotas) {
        // 校验本身配额是否合法 配额量>真实使用量
        if (!ListTool.isEmpty(oldQuotas)) {
            Map<String, ResourceQuota> oldQuotaMap = Maps.newHashMap();
            for (ResourceQuota oldQuota : oldQuotas) {
                oldQuotaMap.put(oldQuota.getCode(), oldQuota);
            }
            for (ResourceQuota newQuota : newQuotas) {
                ResourceQuota oldQuota = oldQuotaMap.get(newQuota.getCode());
                if (oldQuota != null) {
                    Long used = Optional.ofNullable(oldQuota.getUsed()).orElse(0L);
                    if (newQuota.getQuota() < used) {
                        return new Result(false, newQuota.getName() + "配额不能小于已用配额");
                    }
                }
            }
        }
        return new Result(true, "校验通过");
    }

    /**
     * 新增或者更新配额
     *
     * @param oldQuotas
     * @param newQuotas
     * @param requestContext
     * @return
     */
    private Result process(List<ResourceQuota> oldQuotas, List<ResourceQuota> newQuotas, RequestContext requestContext) {
        // 新增配额集合
        List<ResourceQuota> insertQuotas = Lists.newArrayList();
        // 更新配额集合
        List<ResourceQuota> updateQuotas = Lists.newArrayList();
        // 配额记录集合
        List<QuotaHistory> histories = Lists.newArrayList();
        if (ListTool.isEmpty(oldQuotas)) {
            // 原配额为空，全部是新增操作
            for (ResourceQuota quota : newQuotas) {
                quota.setUsed(0L);
                insertQuotas.add(quota);
                // 新增配额记录
                if (quota.getQuota() != null && quota.getQuota() > 0) {
                    QuotaHistory history =
                            new QuotaHistory(quota.getTarget(), quota.getQuota(), quota.getCode(), quota.getUnit(),
                                    quota.getCategory(), ResourceQuota.Action.Assign, new Date(), requestContext.getTarget());
                    histories.add(history);
                }
            }
        } else {
            // 原配额不为空 更新操作
            Map<String, ResourceQuota> oldQuotaMap = Maps.newHashMap();
            for (ResourceQuota oldQuota : oldQuotas) {
                oldQuotaMap.put(oldQuota.getCode(), oldQuota);
            }
            for (ResourceQuota quota : newQuotas) {
                ResourceQuota oldQuota = oldQuotaMap.get(quota.getCode());
                // 不存在旧配额 新增配额
                if (null == oldQuota) {
                    quota.setQuota(Optional.ofNullable(quota.getQuota()).orElse(0L));
                    insertQuotas.add(quota);
                    Long quotaValue = quota.getQuota();
                    if (quotaValue != null && quotaValue > 0) {
                        // 目标租户新增配额记录
                        QuotaHistory assignHistory =
                                new QuotaHistory(quota.getTarget(), quotaValue, quota.getCode(), quota.getUnit(),
                                        quota.getCategory(), ResourceQuota.Action.Assign, new Date(),
                                        requestContext.getTarget());
                        histories.add(assignHistory);
                    }
                } else {
                    // 存在旧配额 更新配额操作
                    Long oldQuotaValue = Optional.ofNullable(oldQuota.getQuota()).orElse(0L);
                    Long newQuotaValue = Optional.ofNullable(quota.getQuota()).orElse(0L);
                    long changeValue = newQuotaValue - oldQuotaValue;
                    oldQuota.setOldQuota(oldQuotaValue);
                    oldQuota.setQuota(newQuotaValue);
                    updateQuotas.add(oldQuota);
                    if (changeValue != 0L) {
                        // 旧配额修改记录
                        QuotaHistory updateHistory =
                                new QuotaHistory(oldQuota.getTarget(), changeValue > 0 ? changeValue : -changeValue,
                                        quota.getCode(), quota.getUnit(), quota.getCategory(),
                                        changeValue > 0 ? ResourceQuota.Action.Assign : ResourceQuota.Action.Decrease,
                                        new Date(), requestContext.getTarget());
                        histories.add(updateHistory);
                    }
                }
            }
        } // 要新增的配额
        if (!ListTool.isEmpty(insertQuotas)) {
            insertQuotas.forEach(quotaRepository::save);
        }
        // 要更新的配额
        if (!ListTool.isEmpty(updateQuotas)) {
            String[] filed = {"id", "quota", "assigned", "used"};
            quotaRepository.batchUpdate(updateQuotas, filed, updateQuotas.size());
        }
        // 配额记录
        if (!ListTool.isEmpty(histories)) {
            histories.forEach(quotaHistoryRepository::save);
        }
        return new Result(true, "配额成功");
    }

    private GeneralResult<List<ResourceQuota>> listTenantQuotas(Long tenantId) {
        List<ResourceQuota> list = quotaRepository.listByTenantId(tenantId);
        List<ResourceQuota> resultQuota = new ArrayList<>();
        License content = LicenseDaemon.instance(currentService).content();
        Map<String, CloudService> serviceMap = cloudServiceRepository.listByStatus(null, content.getPlugins(), content.getGlobals())
                .stream().collect(toMap(CloudService::getCode, Function.identity()));
        Map<ResourceQuota.Category, List<ResourceQuota>> quotaMap = list.stream().collect(groupingBy(ResourceQuota::getCategory));
        //构建基础配额
        List<ResourceQuota> mateQuotas = Optional.ofNullable(quotaMap.get(ResourceQuota.Category.Meta)).orElse(new ArrayList<>());
        for (ResourceQuota mateQuota : mateQuotas) {
            if (mateQuota != null) {
                for (QuotaCategory quotaCategory : QuotaCategory.values()) {
                    if (quotaCategory.getCode().equals(mateQuota.getCode())) {
                        mateQuota.setName(quotaCategory.getName());
                        mateQuota.setSn(0L);
                    }
                }
                mateQuota.setFreeQuota(mateQuota.getQuota() - mateQuota.getUsed());
                resultQuota.add(mateQuota);
            }
        }
        //构建服务配额
        List<ResourceQuota> serviceQuotas =
                Optional.ofNullable(quotaMap.get(ResourceQuota.Category.Service)).orElse(new ArrayList<>());
        for (ResourceQuota serviceQuota : serviceQuotas) {
            if (null != serviceQuota) {
                CloudService service = serviceMap.get(serviceQuota.getCode());
                if (null == service) {
                    log.warn("service not found, code: {}", serviceQuota.getCode());
                    continue;
                }
                serviceQuota.setName(service.getName());
                serviceQuota.setSn(service.getCatalogId() + Long.parseLong(service.getSn()));
                serviceQuota.setFreeQuota(serviceQuota.getQuota() - serviceQuota.getUsed());
                resultQuota.add(serviceQuota);
            }
        }
        resultQuota.sort((first, second) -> {
            if (null == first || second == null) {
                return 0;
            }
            return first.getSn().compareTo(second.getSn());
        });
        return new GeneralResult<>(true, resultQuota, "查询配额成功");
    }

    @Override
    public Result quotaAction(List<ResourceQuota> quotas, ResourceQuota.Action action, Long tenantId) {
        log.info("quotaAction, action:" + action + ", tenantId: " + tenantId + "quotas: " + JSON.toJSONString(quotas));
        String path = "Console_quota_update" + "_" + tenantId;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Tenant tenantInfo = smsTenantService.detail(tenantId);
            if (!tenantInfo.getQuotaAble()) {
                return new Result(true, "租户未开启配额，无需扣除");
            }
            List<QuotaHistory> histories = new ArrayList<>();
            List<ResourceQuota> updateQuotas = new ArrayList<>();
            if (ResourceQuota.Action.Release.equals(action)) {
                for (ResourceQuota quota : quotas) {
                    if (quota.getQuotaActionValue() != null && quota.getQuotaActionValue() != 0L) {
                        QuotaHistory quotaHistory = new QuotaHistory(quota, action, tenantId);
                        histories.add(quotaHistory);
                    }
                    updateQuotas.add(quota);
                }
            } else if (ResourceQuota.Action.Callback.equals(action) || ResourceQuota.Action.Order.equals(action)) {
                Tenant tenant = smsTenantService.detail(tenantId);
                License content = LicenseDaemon.instance(currentService).content();
                Map<String, String> serviceMap = cloudServiceRepository.listByStatus("", content.getPlugins(), content.getGlobals()).stream()
                        .collect(toMap(CloudService::getCode, CloudService::getName));
                Assert.notNull(tenant, "租户信息不存在");
                log.info("The quota of tenant {} starts to {}", tenant.getName(), action.name().toLowerCase());
                // 获取租户配额信息
                Map<String, ResourceQuota> tenantQuotas = quotaRepository.listByTenantId(tenantId).stream()
                        .collect(Collectors.toMap(ResourceQuota::getCode, Function.identity(), (m1, m2) -> m2));
                for (ResourceQuota quota : quotas) {
                    String quotaName;
                    if (QuotaCategory.CPU.getCode().equals(quota.getCode())) {
                        quotaName = QuotaCategory.CPU.getName();
                    } else if (QuotaCategory.MEM.getCode().equals(quota.getCode())) {
                        quotaName = QuotaCategory.MEM.getName();
                    } else if (QuotaCategory.DISK.getCode().equals(quota.getCode())) {
                        quotaName = QuotaCategory.DISK.getName();
                    } else if (QuotaCategory.SNAPSHOT.getCode().equals(quota.getCode())) {
                        quotaName = QuotaCategory.SNAPSHOT.getName();
                    } else {
                        quotaName = serviceMap.get(quota.getCode());
                    }
                    ResourceQuota tenantQuota = tenantQuotas.get(quota.getCode());
                    if (tenantQuota == null) {
                        throw new RuntimeException(quotaName + "配额不足，操作失败。");
                    }
                    long freeQuota = tenantQuota.getQuota() - tenantQuota.getUsed();
                    //订购时 可用配额需要大于扣除配额
                    if (ResourceQuota.Action.Order.equals(action) && freeQuota < quota.getQuotaActionValue()) {
                        throw new RuntimeException(quotaName + "配额不足，操作失败。");
                    }
                    Long actionValue = Optional.ofNullable(quota.getQuotaActionValue()).orElse(0L);
                    // 租户配额 记录操作量
                    tenantQuota.setQuotaActionValue(actionValue);
                    if (actionValue > 0L) {
                        QuotaHistory quotaHistory =
                                new QuotaHistory(tenantQuota.getTarget(), actionValue, tenantQuota.getCode(),
                                        tenantQuota.getUnit(), tenantQuota.getCategory(), action, new Date(), tenantId);
                        histories.add(quotaHistory);
                    }
                    if (ResourceQuota.Action.Callback.equals(action)) {
                        tenantQuota.setUsed(tenantQuota.getUsed() - actionValue);
                    } else {
                        tenantQuota.setUsed(tenantQuota.getUsed() + actionValue);
                    }
                    updateQuotas.add(tenantQuota);
                }
            }
            if (!ListTool.isEmpty(updateQuotas)) {
                String[] filed = {"id", "quota", "assigned", "used"};
                log.info("quota data: " + JSONObject.toJSONString(updateQuotas));
                quotaRepository.batchUpdate(updateQuotas, filed, updateQuotas.size());
            }
            if (!ListTool.isEmpty(histories)) {
                histories.forEach(quotaHistoryRepository::save);
            }
            return new Result(true, "配额更新成功");
        }
    }

    @Override
    public GeneralResult<List<ResourceQuota>> quotas(Long id, String type, RequestContext context) {
        Tenant tenant = smsTenantService.detail(id);
        if (context.getCatalog().equals(Catalog.Manager)) {
            if (StringUtils.isNotBlank(type)) {
                GeneralResult<List<ResourceQuota>> resourceQuotas = listTenantQuotas(id);
                if (resourceQuotas.isFailed()) {
                    return resourceQuotas;
                } else {
                    List<ResourceQuota> data = resourceQuotas.getData();
                    JSONArray jsonArray = new JSONArray();
                    for (ResourceQuota quota : data) {
                        JSONObject quotaObject = new JSONObject();
                        quotaObject.put("name", quota.getName());
                        quotaObject.put("unit", quota.getUnit());
                        JSONArray quotaArray = new JSONArray();
                        JSONObject used = new JSONObject();
                        used.put("name", "已使用");
                        used.put("value", quota.getUsed());
                        quotaArray.add(used);
                        JSONObject free = new JSONObject();
                        free.put("name", "未使用");
                        free.put("value", quota.getFreeQuota());
                        quotaArray.add(free);
                        quotaObject.put("data", quotaArray);
                        jsonArray.add(quotaObject);
                    }
                    return new GeneralResult(true, jsonArray, "查询成功");
                }
            } else {
                return listTenantQuotas(id);
            }
        } else {
            if (Optional.ofNullable(tenant.getQuotaAble()).orElse(false)) {
                return listQuotaDetail(id);
            } else {
                //没开启配额
                return new GeneralResult(true, "查询成功");
            }
        }
    }

    /**
     * 获取服务目录下的配额详情
     *
     * @param tenantId
     * @return
     */
    private GeneralResult listQuotaDetail(Long tenantId) {
        try {
            List<CloudService> tenantService = cloudServiceRepository.listTenantService(tenantId).stream()
                    .filter(cloudService -> StringUtils.isNotEmpty(cloudService.getUnit())).collect(Collectors.toList());
            HashMap<String, QuotaModel> quotaMap = new HashMap<>();
            Map<String, ResourceQuota> mateQuotaMap =
                    quotaRepository.list(tenantId, ResourceQuota.Category.Meta.name(), null).stream()
                            .collect(toMap(ResourceQuota::getCode, Function.identity()));

            List<QuotaModel.Quota> resourceQuotas = new ArrayList<>();
            for (Map.Entry<String, ResourceQuota> quotaEntry : mateQuotaMap.entrySet()) {
                String code = quotaEntry.getKey();
                ResourceQuota quota = quotaEntry.getValue();
                QuotaModel.Quota.QuotaDetail resourceQuotaData = new QuotaModel.Quota.QuotaDetail();
                if (QuotaCategory.CPU.getCode().equals(code)) {
                    resourceQuotaData.setQuotaName(QuotaCategory.CPU.getName());
                } else if (QuotaCategory.MEM.getCode().equals(code)) {
                    resourceQuotaData.setQuotaName(QuotaCategory.MEM.getName());
                } else if (QuotaCategory.DISK.getCode().equals(code)) {
                    resourceQuotaData.setQuotaName(QuotaCategory.DISK.getName());
                } else if (QuotaCategory.SNAPSHOT.getCode().equals(code)) {
                    resourceQuotaData.setQuotaName(QuotaCategory.SNAPSHOT.getName());
                }
                resourceQuotaData.setUsed(quota.getUsed());
                resourceQuotaData.setQuota(quota.getQuota());
                resourceQuotaData.setFree(quota.getQuota() - quota.getUsed());
                resourceQuotaData.setUnit(quota.getUnit());
                QuotaModel.Quota resourceQuota = new QuotaModel.Quota();
                resourceQuota.setQuotaDetail(resourceQuotaData);
                resourceQuotas.add(resourceQuota);
            }
            QuotaModel resourceModel = new QuotaModel();
            resourceModel.setCatalogName("基础配额");
            resourceModel.setSn("0");
            resourceModel.setQuota(resourceQuotas);
            quotaMap.put(resourceModel.getCatalogName(), resourceModel);
            for (CloudService service : tenantService) {
                ResourceQuota quota = quotaRepository.query(tenantId, service.getCode());
                QuotaModel.Quota.QuotaDetail quotaData = null;
                if (quota != null) {
                    quotaData = new QuotaModel.Quota.QuotaDetail();
                    quotaData.setQuotaName(service.getName());
                    quotaData.setUsed(quota.getUsed());
                    quotaData.setQuota(quota.getQuota());
                    quotaData.setFree(quota.getQuota() - quota.getUsed());
                    quotaData.setUnit(quota.getUnit());
                }
                QuotaModel.Quota serviceQuota = new QuotaModel.Quota();
                serviceQuota.setServiceName(service.getName());
                serviceQuota.setQuotaDetail(quotaData);
                List<QuotaModel.Quota> serviceQuotas = new ArrayList<>();
                if (quotaMap.containsKey(service.getCatalogName())) {
                    serviceQuotas = quotaMap.get(service.getCatalogName()).getQuota();
                }
                serviceQuotas.add(serviceQuota);
                QuotaModel quotaModel = new QuotaModel();
                if (quotaMap.containsKey(service.getCatalogName())) {
                    quotaModel = quotaMap.get(service.getCatalogName());
                }
                quotaModel.setCatalogName(service.getCatalogName());
                quotaModel.setSn(serviceCatalogRepository.get(ServiceCatalog.class, service.getCatalogId()).getSn());
                quotaModel.setQuota(serviceQuotas);
                quotaMap.put(quotaModel.getCatalogName(), quotaModel);
            }
            ArrayList<QuotaModel> quotaModels = new ArrayList<>(quotaMap.values());
            quotaModels.sort((first, second) -> {
                if (null == first || second == null) {
                    return 0;
                }
                return first.getSn().compareTo(second.getSn());
            });
            return new GeneralResult<>(true, quotaModels, "查询成功");
        } catch (Exception e) {
            log.error("get quota info error:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 检查是否存在服务配额使用
     *
     * @param id 租户id
     * @return
     */
    @Override
    public Result checkServiceQuota(Long id) {
        try {
            List<ResourceQuota> list = quotaRepository.list(id, ResourceQuota.Category.Service.name(), null);
            for (ResourceQuota serviceQuota : list) {
                // 配额使用不可释放(不考虑子租户分配)
                if (serviceQuota.getUsed() > 0) {
                    return new Result(false, "租户存在资源使用");
                }
            }
            return new Result(true, "租户无资源使用情况");
        } catch (Exception e) {
            log.error("list resource quota fail:", e);
            return new Result(false, "查询配额失败");
        }
    }

    /**
     * 获取创建时资源配额范围
     *
     * @param requestContext
     * @return
     */
    @Override
    public GeneralResult<List<ResourceQuota>> getMetaQuota(RequestContext requestContext) {
        try {
            // 自服务端请求需要受所属租户限制
            Map<String, ResourceQuota> quotaLimit = new HashMap<>();
            if (Catalog.Tenant.equals(requestContext.getCatalog())) {
                quotaLimit =
                        quotaRepository.list(requestContext.getTarget(), ResourceQuota.Category.Meta.name(), null).stream()
                                .collect(Collectors.toMap(ResourceQuota::getCode, Function.identity()));
            }
            ArrayList<ResourceQuota> quotas = new ArrayList<>();
            for (QuotaCategory quotaCategory : QuotaCategory.values()) {
                ResourceQuota quota = new ResourceQuota(null, 0L, 0L, quotaCategory.getUnit(), quotaCategory.getCode(),
                        ResourceQuota.Category.Meta, 0L, 99999999L, quotaCategory.getName());
                // 是否有配额限制
                ResourceQuota limitQuota = quotaLimit.get(quotaCategory.getCode());
                if (limitQuota != null) {
                    //租户剩余可用的配额=总配额-已分配配额
                    quota.setMaxQuota(limitQuota.getQuota() - limitQuota.getUsed());
                }
                quotas.add(quota);
            }
            return new GeneralResult<>(true, quotas, "获取基础配额成功。");
        } catch (Exception e) {
            log.error("get mate quota data error", e);
            return new GeneralResult<>(false, "获取基础配额失败。");
        }
    }

    @Override
    public GeneralResult<List<ResourceQuota>> getTenantMetaQuota(Long id, RequestContext requestContext) {
        try {
            Tenant tenant = smsTenantService.detail(id);
            Assert.notNull(tenant, "租户不存在");
            //查询当前租户配额
            Map<String, ResourceQuota> tenantQuotaMap =
                    quotaRepository.list(tenant.getId(), ResourceQuota.Category.Meta.name(), null).stream()
                            .collect(toMap(ResourceQuota::getCode, Function.identity()));
            List<ResourceQuota> quotas = new ArrayList<>();
            for (QuotaCategory quotaCategory : QuotaCategory.values()) {
                ResourceQuota quota = new ResourceQuota(null, 0L, 0L, quotaCategory.getUnit(), quotaCategory.getCode(),
                        ResourceQuota.Category.Meta, 0L, 99999999L, quotaCategory.getName());
                //自身使用量限制配额最小值
                ResourceQuota tenantQuota = tenantQuotaMap.get(quotaCategory.getCode());
                if (tenantQuota != null) {
                    quota.setMinQuota(tenantQuota.getUsed() < 0 ? 0 : tenantQuota.getUsed());
                    quota.setQuota(tenantQuota.getQuota() < 0 ? 0 : tenantQuota.getQuota());
                }
                quotas.add(quota);
            }
            return new GeneralResult<>(true, quotas, "获取元数据配额成功。");
        } catch (Exception e) {
            log.error("get mate quota data error", e);
            return new GeneralResult<>(false, "获取元数据配额失败。");
        }
    }

    @Override
    public GeneralResult<List<ResourceQuota>> handleQuota(Map<String, Integer> quotaMap, Long tenantId, boolean isCheck) {
        Tenant tenant = tenantService.detail(tenantId);
        if (!tenant.getQuotaAble()) {
            return new GeneralResult(true, "租户未开启配额，无需扣除");
        }
        if (null == quotaMap || quotaMap.isEmpty()) {
            // 无配额信息需更新
            log.debug("quotaMap is null or empty! 无配额信息需更新");
        }
        List<ResourceQuota> list = quotaRepository.listByTenantId(tenantId);
        Assert.notEmpty(list, "未查询到配额信息");
        Map<String, ResourceQuota> map = new HashMap<>(8);
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (Map.Entry<String, Integer> entry : quotaMap.entrySet()) {
            String code = entry.getKey();
            String quotaName;
            if (QuotaCategory.CPU.getCode().equals(code)) {
                quotaName = QuotaCategory.CPU.getName();
            } else if (QuotaCategory.MEM.getCode().equals(code)) {
                quotaName = QuotaCategory.MEM.getName();
            } else if (QuotaCategory.DISK.getCode().equals(code)) {
                quotaName = QuotaCategory.DISK.getName();
            } else {
                CloudService cloudService = cloudServiceRepository.queryByCode(code);
                if (StringUtils.isBlank(cloudService.getUnit())) {
                    continue;
                }
                quotaName = cloudService.getName();
            }
            if (!map.containsKey(entry.getKey())) {
                throw new RuntimeException("未查询到" + quotaName + "配额信息");
            }
            ResourceQuota quota = map.get(entry.getKey());
            if (entry.getValue() + quota.getUsed() > quota.getQuota()) {
                throw new RuntimeException("剩余" + quotaName + "配额不足");
            }
            quota.setQuotaActionValue(entry.getValue().longValue());
            quotas.add(quota);
        }
        if (isCheck) {
            return new GeneralResult(true, quotas, "校验配额通过");
        }
        log.info("要扣除的配额数量：" + JSON.toJSONString(quotas));
        Result result = quotaAction(quotas, ResourceQuota.Action.Order, tenantId);
        if (result.isFailed()) {
            return new GeneralResult(false, result.getMessage());
        }
        return new GeneralResult(true, null, "更新配额信息成功");
    }

    /**
     * 回滚配额
     *
     * @param beans    配额集合
     * @param tenantId 租户ID
     */
    @Override
    public void callBackQuota(List<QuotaBean> beans, Long tenantId) {
        List<ResourceQuota> list = quotaRepository.listByTenantId(tenantId);
        Map<String, ResourceQuota> map = new HashMap<>();
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (QuotaBean bean : beans) {
            if (!map.containsKey(bean.getCode())) {
                continue;
            }
            ResourceQuota quota = map.get(bean.getCode());
            // 记录回滚配额操作量
            quota.setQuotaActionValue(bean.getValue().longValue());
            quotas.add(quota);
        }
        this.quotaAction(quotas, ResourceQuota.Action.Callback, tenantId);
    }
}
