package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.internel.*;
import com.bocloud.cos.enums.BocloudStatus;
import com.bocloud.cos.enums.VendorType;
import com.bocloud.cos.service.internal.ImsInternalService;
import com.bocloud.cos.intf.ResourceScreenService;
import com.bocloud.cos.model.ChargeResultTrend;
import com.bocloud.cos.model.ChargeTrendData;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.utils.DateUtils;
import com.bocloud.cos.utils.MathUtils;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 平台资源大屏业务类
 *
 * @author tyl
 * @since 2020/11/25 21:49
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class ResourceScreenServiceImpl implements ResourceScreenService {

    private final ImsInternalService imsInternalService;

    @Override
    public GeneralResult totalOverview(RequestContext requestContext) {
        JSONObject result = new JSONObject();
        try {
            List<CloudServer> cloudServers = imsInternalService.listVmsByVendor(null, false, requestContext);
            long running =
                    cloudServers.stream().filter(cloudServer -> "RUNNING".equals(cloudServer.getStatus())).count();
            List<Server> servers = imsInternalService.listHosts();
            result.put("hosts", servers.size());
            result.put("servers", cloudServers.size());
            result.put("running", running);
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get total overview error,", e);
            return new GeneralResult<>(false, result, "获取数据中心总体情况失败");
        }
    }

    @Override
    public GeneralResult platformOverview(RequestContext requestContext) {
        JSONObject result = new JSONObject();
        try {
            List<CloudVendor> vendorList = imsInternalService.getVendorList();
            Map<Boolean, List<CloudVendor>> collect = vendorList.stream().collect(Collectors.groupingBy(CloudVendor::getIsPublic));
            List<CloudVendor> priVendor = Optional.ofNullable(collect.get(false)).orElse(new ArrayList<>());
            List<CloudVendor> pubVendor = Optional.ofNullable(collect.get(true)).orElse(new ArrayList<>());
            result.put("priNum", priVendor.size());
            result.put("pubNum", pubVendor.size());
            return new GeneralResult<>(true, result, "获取平台情况成功");
        } catch (Exception e) {
            log.error("get platform overview error,", e);
            return new GeneralResult<>(false, result, "获取平台情况失败");
        }
    }

    @Override
    public GeneralResult usedOverview(RequestContext requestContext) {
        JSONObject result = new JSONObject();
        try {
            //私有云平台
            List<CloudVendor> cloudVendors = imsInternalService.listCloudVendor(null);
            Set<String> vendors = cloudVendors.stream().filter(cloudVendor -> !cloudVendor.getIsPublic())
                    .map(CloudVendor::getType).collect(Collectors.toSet());
            List<Server> hosts = imsInternalService.listHosts();
            hosts.removeIf(server -> null == server.getVendorId());
            List<Server> opServers = hosts.stream().filter(server -> null != server.getVendorType())
                    .filter(server -> vendors.contains(server.getVendorType())).collect(Collectors.toList());
            //计算总量
            long cpuTotal = 0L;
            long memTotal = 0L;
            int diskTotal = 0;
            for (Server opServer : opServers) {
                ServerConfig config = opServer.getConfig();
                if (null == config) {
                    continue;
                }
                cpuTotal += Optional.ofNullable(config.getCpuSum()).orElse(0).longValue();
                diskTotal += Optional.ofNullable(config.getDiskCapacity()).orElse(0D).intValue();
                memTotal += Optional.ofNullable(config.getMemoryCapacity()).orElse(0D).longValue();
            }
            result.put("cpuTotal", cpuTotal);
            result.put("memTotal", memTotal);
            result.put("diskTotal", MathUtils.round(diskTotal / 1024D, 2));
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get resource overview failed: {}", e.getMessage(), e);
            return new GeneralResult<>(false, result, "获取资源总量失败");
        }
    }

    @Override
    public GeneralResult count(String type, RequestContext requestContext) {
        JSONObject result = new JSONObject();
        try {
            //云平台
            List<CloudVendor> cloudVendors = imsInternalService.listCloudVendor(null);
            Set<String> vendors =
                    cloudVendors.stream()
                            .map(CloudVendor::getType).collect(Collectors.toSet());
            List<CloudServer> cloudServerList = imsInternalService.listVmsByVendor(type, false, requestContext).stream()
                    .filter(c -> null != c.getVendorType() && vendors.contains(c.getVendorType()))
                    .collect(Collectors.toList());
            Integer cpuNum = cloudServerList.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu())
                    .orElse(0)).reduce(Integer::sum).orElse(0);
            BigDecimal memNum = cloudServerList.stream()
                    .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            Integer diskNum = cloudServerList.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getDisk())
                    .orElse(0)).reduce(Integer::sum).orElse(0);
            result.put("serverNum", cloudServerList.size());
            result.put("cpu", cpuNum);
            result.put("mem", memNum.doubleValue());
            result.put("disk", MathUtils.round(diskNum / 1024D, 2));
            return new GeneralResult<>(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get vendor resource count error ", e);
            return new GeneralResult<>(false, result, "查询平台云资源统计失败:" + e.getMessage());
        }
    }

    private List<Volume> getVolumes(List<CloudServer> cloudServerList) {
        List<Volume> volumes = new ArrayList<>();
        if (ListTool.isEmpty(cloudServerList)) {
            return volumes;
        }
        //单独处理云硬盘，可能出现数据盘的情况
        //取出CloudServer下所有的vendorId
        List<Long> cloudVendorIds = cloudServerList.stream().map(CloudServer::getVendorId).distinct().collect(Collectors.toList());
        //筛选指定云主机下的磁盘--由于存在数据盘，所以改用取volume信息，没有用server对应的disk信息
        List<Long> serverIds = cloudServerList.stream().map(CloudServer::getId).collect(Collectors.toList());
        for (Long vendorId : cloudVendorIds) {
            GeneralResult serviceVolume = imsInternalService.getVolume(vendorId);
            if (serviceVolume.getData() != null) {
                List<Volume> volumeList = JSONArray.parseArray(JSONObject.toJSONString(serviceVolume.getData()), Volume.class);
                volumeList = volumeList.stream().filter(v -> serverIds.contains(v.getServerId())).collect(Collectors.toList());
                volumes.addAll(volumeList);
            }
        }
        return volumes;
    }

    @Override
    public GeneralResult status(String type, RequestContext requestContext) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<CloudServer> cloudServerList = imsInternalService.listVmsByVendor(type, false, requestContext);
            Map<String, List<CloudServer>> serversMap =
                    cloudServerList.stream().collect(Collectors.groupingBy(CloudServer::getStatus));
            List<Object> data = new ArrayList<>();
            for (Map.Entry<String, List<CloudServer>> entry : serversMap.entrySet()) {
                String key;
                if ("RUNNING".equals(entry.getKey()) || "GENERAL".equals(entry.getKey())) {
                    key = "开机";
                } else if ("STOPPED".equals(entry.getKey())) {
                    key = "关机";
                } else if ("ERROR".equals(entry.getKey()) || "UNKNOWN".equals(entry.getKey())) {
                    key = "断开";
                } else if ("EXCEPTION".equals(entry.getKey())) {
                    key = "异常";
                } else if ("SUSPENDED".equals(entry.getKey())) {
                    key = "挂起";
                } else {
                    continue;
                }
                int count = entry.getValue().size();
                keys.add(key);
                data.add(count);
            }
            values.add(new ChargeTrendData("云主机数量（个）", data));
            return new GeneralResult(true, new ChargeResultTrend(keys, values), "查询云主机状态统计成功");
        } catch (Exception e) {
            log.error("get vendor resource count error ", e);
            return new GeneralResult(false, new ChargeResultTrend(keys, values), "查询云主机状态统计失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult vendorTypes(RequestContext requestContext) {
        JSONArray result = new JSONArray();
        try {
            List<CloudVendor> vendors = imsInternalService.listCloudVendor(null);
            Set<String> collect = vendors.stream().map(CloudVendor::getType).collect(Collectors.toSet());
            for (VendorType type : VendorType.values()) {
                for (String vendorType : collect) {
                    if (type.name().equals(vendorType)) {
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put("name", type.getName());
                        jsonObject.put("value", vendorType);
                        result.add(jsonObject);
                        break;
                    }
                }
            }
            return new GeneralResult(true, result, "查询成功");
        } catch (Exception e) {
            log.error("get vendor types error ", e);
            return new GeneralResult(false, result, "查询平台类型失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult resourceApply(String type, RequestContext requestContext) {
        List<String> timeList = DateUtil.timeToList(new JSONObject());
        List<String> keys = new ArrayList<>(timeList);
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<CloudServer> cloudServerList = imsInternalService.listVmsByVendor(type, false, requestContext);
            Set<String> vendorTypes = imsInternalService.listCloudVendor(type).stream().map(CloudVendor::getType)
                    .filter(s -> !s.isEmpty()).collect(Collectors.toSet());
            HashMap<String, List<Object>> dataMap = new HashMap<>(8);
            for (String vendorType : vendorTypes) {
                dataMap.put(vendorType, new ArrayList<>());
            }
            for (String time : timeList) {
                //筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
                List<CloudServer> targetVms = cloudServerList.stream().filter(
                                cloudServer -> !BocloudStatus.Server.BUILDING.name().equals(cloudServer.getStatus()) && cloudServer
                                        .getGmtCreate()
                                        .before(DateUtils.stringToDate(time.substring(0, 10) + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                                        && cloudServer.getGmtCreate()
                                        .after(DateUtils.stringToDate(time.substring(0, 10) + " 00:00:00", "yyyy-MM-dd HH:mm:ss")))
                        .collect(Collectors.toList());
                Map<String, List<CloudServer>> vmsMap =
                        targetVms.stream().filter(cloudServer -> cloudServer.getVendorType() != null)
                                .collect(Collectors.groupingBy(CloudServer::getVendorType));
                for (Map.Entry<String, List<Object>> entry : dataMap.entrySet()) {
                    String vendorType = entry.getKey();
                    List<Object> data = entry.getValue();
                    List<CloudServer> servers = vmsMap.get(vendorType);
                    Integer serverNumber = servers != null ? servers.size() : 0;
                    data.add(serverNumber);
                }
            }
            for (Map.Entry<String, List<Object>> entry : dataMap.entrySet()) {
                String key = entry.getKey();
                List<Object> data = entry.getValue();
                VendorType vendorType = VendorType.valueOf(key);
                values.add(new ChargeTrendData(vendorType.getName(), data));
            }
            return new GeneralResult<>(true, new ChargeResultTrend(keys, values), "查询云主机申请趋势成功");
        } catch (Exception e) {
            log.error("get current vendor resource apply info error,", e);
            return new GeneralResult<>(false, new ChargeResultTrend(keys, values), "查询云主机申请趋势失败");
        }
    }

    @Override
    public GeneralResult cloudResourceTrend(String type, RequestContext requestContext) {
        List<String> timeList = DateUtil.timeToList(new JSONObject());
        List<String> keys = new ArrayList<>(timeList);
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<CloudServer> cloudServerList = imsInternalService.listVmsByVendor(type, false, requestContext);
            List<Volume> volumes = getVolumes(cloudServerList);
            List<Object> cpuData = new ArrayList<>();
            List<Object> memData = new ArrayList<>();
            List<Object> diskData = new ArrayList<>();
            for (String time : timeList) {
                //筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
                List<CloudServer> targetVms = cloudServerList.stream().filter(
                                vm -> !BocloudStatus.Server.BUILDING.name().equals(vm.getStatus()) && vm
                                        .getGmtCreate()
                                        .before(DateUtils.stringToDate(time.substring(0, 10) + " 23:59:59", "yyyy-MM-dd HH:mm:ss")))
                        .collect(Collectors.toList());
                Integer cpuNum = targetVms.stream().map(vm -> Optional.ofNullable(vm.getCpu()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                cpuData.add(cpuNum);
                BigDecimal memNum = targetVms.stream().map(vm -> Optional.ofNullable(vm.getMemory()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                memData.add(memNum.doubleValue());
                //筛选指定云主机下的磁盘--由于存在数据盘，所以改用取volume信息，没有用server对应的disk信息
                List<Long> serverIds = targetVms.stream().map(CloudServer::getId).collect(Collectors.toList());
                List<Volume> volumeList = volumes.stream().filter(volume -> serverIds.contains(volume.getServerId()))
                        .collect(Collectors.toList());
                Integer diskNum = volumeList.stream().map(volume -> Optional.ofNullable(volume.getSize()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                diskData.add(diskNum);
            }
            values.add(new ChargeTrendData("CPU(C)", cpuData));
            values.add(new ChargeTrendData("内存(GB)", memData));
            values.add(new ChargeTrendData("磁盘(GB)", diskData));
            return new GeneralResult<>(true, new ChargeResultTrend(keys, values), "查询公有云平台资源使用趋势成功");
        } catch (Exception e) {
            log.error("get current vendor resource used info error,", e);
            return new GeneralResult<>(false, new ChargeResultTrend(keys, values), "查询公有云平台资源使用趋势失败");
        }
    }
}
