package com.bocloud.cos.service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import com.megatron.common.model.*;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.license.LicenseDaemon;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.cos.entity.ServiceCategory;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.intf.ServiceCategoryService;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.OrderSnRepository;
import com.bocloud.cos.repository.ServiceCatalogRepository;
import com.bocloud.cos.repository.ServiceCategoryRepository;
import com.bocloud.cos.repository.ServiceSkuRepository;
import com.google.common.collect.Lists;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * 服务类型业务操作实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class ServiceCategoryServiceImpl implements ServiceCategoryService, InitializingBean {

    private final ServiceCategoryRepository serviceCategoryRepository;
    private final ServiceCatalogRepository catalogRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final LockFactory lockFactory;
    private final OrderSnRepository snRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final StringRedisTemplate redisTemplate;
    private final CurrentService currentService;

    @Override
    public GeneralResult<GridBean<ServiceCategory>> list(Pager pager) {
        int total = this.serviceCategoryRepository.count(pager.getParams());
        List<ServiceCategory> list = this.serviceCategoryRepository.list(pager);
        GridBean<ServiceCategory> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        return new GeneralResult<>(true, gridBean, "查询成功");
    }

    @Override
    public Result remove(Long id, Long userId) {
        ServiceSku serviceSku = serviceSkuRepository.getByCategoryId(id);
        if (serviceSku != null && serviceSku.getSpec() != null) {
            return new Result(false, "该服务类型下存在规格无法删除");
        }
        Boolean result = serviceCategoryRepository.remove(id, userId);
        if (result) {
            snRepository.removeByCategoryId(id);
            redisTemplate.delete("ServiceCategory_" + id);
        }
        return new Result(true, "删除成功");
    }

    @Override
    public GeneralResult<ServiceCategory> detail(Long id) {
        ServiceCategory serviceCategory = serviceCategoryRepository.query(id);
        return new GeneralResult<>(true, serviceCategory, "查询成功");
    }

    @Override
    public Result create(ServiceCategory category, RequestContext context) {
        try {
            category.setCreatorId(context.getTarget());
            category.setMenderId(context.getTarget());
            boolean flag = serviceCategoryRepository.existsName(category.getName(), category.getServiceId());
            if (flag) {
                return new Result(false, "类型名称已存在");
            }
            flag = serviceCategoryRepository.existsSn(category.getSn(), category.getServiceId());
            if (flag) {
                return new Result(false, "当前服务下的类型编号已存在");
            }
            category.setReserved(false);
            Integer maxId = serviceCategoryRepository.getMaxId();
            if (null == maxId) {
                maxId = 0;
            }
            category.setSn(String.format("%02d", maxId));
            serviceCategoryRepository.save(category);
            redisTemplate.opsForValue().set("ServiceCategory_" + category.getId(), category.getName());
            return new Result(true, "新增成功");
        } catch (Exception e) {
            log.error("Query error:", e);
            return new Result(false, "新增失败");
        }
    }

    @Override
    public Result modify(Long id, ServiceCategory category, RequestContext context) {
        String path = ServiceCategory.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceCategory object = serviceCategoryRepository.query(id);
            Assert.notNull(object, "数据不存在！");
            if (!object.getName().equals(category.getName())
                && serviceCategoryRepository.existsName(category.getName(), category.getServiceId())) {
                return new Result(false, "名称已存在");
            }
            if (!object.getSn().equals(category.getSn())
                && serviceCategoryRepository.existsSn(category.getSn(), category.getServiceId())) {
                return new Result(false, "当前服务目录下的云服务sn已存在");
            }
            object.setName(category.getName());
            object.setCode(category.getCode());
            object.setSn(category.getSn());
            object.setRemark(category.getRemark());
            object.setGmtModify(new Date());
            object.setMenderId(context.getTarget());
            serviceCategoryRepository.update(object);
            redisTemplate.opsForValue().set("ServiceCategory_" + object.getId(), object.getName());
            return new Result(true, "修改成功");
        } catch (Exception e) {
            log.error("Modify ServiceCategory fail:", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<List<ServiceCategory>> listByCode(String code) {
        List<ServiceCategory> categories = serviceCategoryRepository.listByServiceCode(code);
        return new GeneralResult<>(true, categories, "查询成功");
    }

    @Override
    public GeneralResult<List<ServiceCategory>> list(List<Param> params) {
        if (null == params) {
            params = Lists.newArrayList();
        }
        List<ServiceCategory> categories = this.serviceCategoryRepository.list(params);
        return new GeneralResult<>(true, categories, "查询成功");
    }

    @Override
    public GeneralResult<JSONArray> tree() {
        List<ServiceCategory> serviceCategories = serviceCategoryRepository.listAll();
        log.info("serviceCategories:" + JSON.toJSONString(serviceCategories));
        Map<String, List<ServiceCategory>> map =
            serviceCategories.stream().filter(serviceCategory -> serviceCategory.getServiceName() != null)
                .collect(Collectors.groupingBy(ServiceCategory::getServiceName));
        // 排序
        List<ServiceCatalog> catalogSortList = catalogRepository.listAll().stream()
            .sorted(Comparator.comparing(ServiceCatalog::getSn)).collect(Collectors.toList());
        log.info("catalogSortList:" + JSON.toJSONString(catalogSortList));
        License content = LicenseDaemon.instance(currentService).content();
        Map<Long, List<CloudService>> serviceMap =
            cloudServiceRepository.listByStatus(CloudService.Status.PUBLISHED.name(),
                    content.getPlugins(),content.getGlobals()).stream()
                .collect(Collectors.groupingBy(CloudService::getCatalogId));
        log.info("serviceMap:" + JSON.toJSONString(serviceMap));
        JSONArray result = new JSONArray();
        ListIterator<ServiceCatalog> catalogListIterator = catalogSortList.listIterator();
        while (catalogListIterator.hasNext()) {
            ServiceCatalog catalog = catalogListIterator.next();
            JSONObject catalogObject = new JSONObject();
            catalogObject.put("id", catalog.getId());
            catalogObject.put("name", catalog.getName());
            JSONArray serviceArray = new JSONArray();
            List<CloudService> cloudServices =
                Optional.ofNullable(serviceMap.get(catalog.getId())).orElse(new ArrayList<>());
            if (!cloudServices.isEmpty()) {
                cloudServices.sort(Comparator.comparing(CloudService::getSn));
                ListIterator<CloudService> cloudServiceListIterator = cloudServices.listIterator();
                while (cloudServiceListIterator.hasNext()) {
                    CloudService cloudService = cloudServiceListIterator.next();
                    JSONObject serviceObject = new JSONObject();
                    serviceObject.put("id", cloudService.getId());
                    serviceObject.put("name", cloudService.getName());
                    List<ServiceCategory> categories =
                        Optional.ofNullable(map.get(cloudService.getName())).orElse(new ArrayList<>());
                    if (!categories.isEmpty()) {
                        categories.sort(Comparator.comparing(ServiceCategory::getSn));
                        serviceObject.put("children", categories);
                    } else {
                        cloudServiceListIterator.remove();
                        continue;
                    }
                    serviceArray.add(serviceObject);
                }
            } else {
                catalogListIterator.remove();
                continue;
            }
            catalogObject.put("children", serviceArray);
            result.add(catalogObject);
        }
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        List<ServiceCategory> categories = serviceCategoryRepository.listAll();
        for (ServiceCategory category : categories) {
            redisTemplate.opsForValue().set("ServiceCategory_" + category.getId(), category.getName());
        }
    }
}
