package com.bocloud.cos.service;

import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ResourceQuota;
import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.cos.entity.ServiceTenant;
import com.bocloud.cos.service.internal.SmsTenantService;
import com.bocloud.cos.service.internal.SmsUserService;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.intf.ServiceTenantService;
import com.bocloud.cos.model.GrantBean;
import com.bocloud.cos.model.MetaBean;
import com.bocloud.cos.model.ServiceBean;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.ResourceQuotaRepository;
import com.bocloud.cos.repository.ServiceCatalogRepository;
import com.bocloud.cos.repository.ServiceTenantRepository;
import com.bocloud.cos.entity.internel.Favorite;
import com.bocloud.cos.entity.internel.Tenant;
import com.google.common.collect.Lists;
import com.megatron.common.exception.InternalServerException;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 服务授权
 *
 * @author Administrator
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class ServiceTenantServiceImpl implements ServiceTenantService {

    private final CloudServiceRepository cloudServiceRepository;
    private final SmsTenantService tenantService;
    private final ResourceQuotaRepository quotaRepository;
    private final LockFactory lockFactory;
    private final SmsUserService smsUserService;
    private final SmsTenantService smsTenantService;
    private final ResourceQuotaService resourceQuotaService;
    private final ServiceCatalogRepository catalogRepository;
    private final ServiceTenantRepository serviceTenantRepository;

    /**
     * 查看租户授权服务详情
     *
     * @param serviceId 服务id
     * @return
     */
    @Override
    public GeneralResult<List<ServiceTenant>> detail(Long serviceId) {
        try {
            List<ServiceTenant> tenants = serviceTenantRepository.listTenant(serviceId);
            return new GeneralResult<>(true, tenants, "查询成功");
        } catch (Exception e) {
            log.error("Query Service Item error:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 服务目录授众
     *
     * @param serviceIds     服务id集合
     * @param tenantIds      租户id集合
     * @param type           操作类型
     * @param requestContext 请求对象
     * @return
     */
    @Override
    @Transactional
    public Result grantBatch(List<Long> serviceIds, List<Long> tenantIds, String type, RequestContext requestContext) {
        if (null == type) {
            return new Result(false, "参数格式错误");
        }
        if (ListTool.isEmpty(serviceIds)) {
            return new Result(false, "请至少选择一个服务进行授权");
        }
        if (ListTool.isEmpty(tenantIds)) {
            return new Result(false, "请至少选择一个租户进行授权");
        }
        String path = ServiceTenant.class.getSimpleName() + "_" + requestContext.getTarget();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Map<Long, Tenant> tenantMap = smsTenantService.listByIds(tenantIds, "NORMAL").stream()
                    .collect(Collectors.toMap(Tenant::getId, Function.identity()));
            switch (type) {
                case "add":
                    return grantServicesToTenants(serviceIds, tenantIds, tenantMap, requestContext);
                case "remove":
                    return removeTenantsServices(serviceIds, tenantIds, requestContext);
                default:
                    return new Result(false, "不支持该操作");
            }
        } catch (Exception e) {
            log.error("Grant Service Item to tenant error:", e);
            throw new InternalServerException("批量操作失败", e, e.getMessage());
        }
    }

    private Result grantServicesToTenants(List<Long> serviceIds, List<Long> tenantIds, Map<Long, Tenant> tenantMap,
                                          RequestContext requestContext) {
        // 用于创建租户与服务关联的集合
        List<ServiceTenant> itemTenants = new ArrayList<>();
        // 用于接收创建收藏的集合
        ArrayList<Favorite> favorites = new ArrayList<>();
        // 用于接收添加配额的集合
        ArrayList<ResourceQuota> addQuota = new ArrayList<>();
        for (Long serviceId : serviceIds) {
            for (Long tenantId : tenantIds) {
                ServiceTenant itemTenant = serviceTenantRepository.getByTenant(tenantId, serviceId);
                if (null == itemTenant) {
                    // 添加租户服务关联集合
                    itemTenants.add(new ServiceTenant(serviceId, tenantId));
                    // 添加租户收藏
                    favorites.add(new Favorite(null, Favorite.Category.Tenant, tenantId, serviceId, 0));
                    CloudService cloudService = cloudServiceRepository.query(serviceId);
                    // 判断是否存在容器服务 存在则添加租户到容器租户集合（后续处理容器租户创建问题）
                    Tenant tenant = tenantMap.get(tenantId);
                    if (tenant.getQuotaAble()) {
                        ResourceQuota quota = quotaRepository.query(tenantId, serviceId);
                        // 如果配额不存在（新增加的服务）则新增配额对象到配额集合
                        if (quota == null && StringUtils.hasText(cloudService.getUnit())) {
                            addQuota.add(
                                    new ResourceQuota(tenantId, 0L, 0L, cloudService.getUnit(), cloudService.getCode(),
                                            ResourceQuota.Category.Service));
                        }
                    }
                }
            }
        }
        // 处理租户服务授权
        if (!ListTool.isEmpty(itemTenants)) {
            itemTenants.forEach(serviceTenantRepository::save);
            // 处理租户服务收藏 收藏历史
            tenantService.addFavorites(favorites);
            if (!addQuota.isEmpty()) {
                Map<Long, List<ResourceQuota>> addQuotaMap =
                        addQuota.stream().collect(Collectors.groupingBy(ResourceQuota::getTarget));
                for (Map.Entry<Long, List<ResourceQuota>> entry : addQuotaMap.entrySet()) {
                    Long tenantId = entry.getKey();
                    List<ResourceQuota> quotas = entry.getValue();
                    resourceQuotaService.assign(tenantId, quotas, requestContext);
                }
            }
        }
        return new Result(true, "授权成功");
    }

    private Result removeTenantsServices(List<Long> serviceIds, List<Long> tenantIds, RequestContext requestContext) {
        for (Long serviceId : serviceIds) {
            for (Long tenantId : tenantIds) {
                ResourceQuota quota = quotaRepository.query(tenantId, serviceId);
                if (quota != null) {
                    if (quota.getUsed() <= 0) {
                        // 资源配额无使用则可以删除
                        resourceQuotaService
                                .delete(tenantId, quota.getCategory().name(), quota.getCode(), requestContext);
                    } else {
                        // 资源配额有使用，不能解除服务授权
                        return new Result(false, "批量操作失败,租户下存在服务配额使用不为空，无法解除服务授权");
                    }
                }
                // 解除该租户的授权服务
                this.cloudServiceRepository.deleteTenant(tenantId, serviceId);
                // 解除租户的服务收藏
                tenantService.cancelFavorite(tenantId, serviceId);
            }
        }
        return new Result(true, "授权成功");
    }

    /**
     * 授权服务
     *
     * @param id             租户id
     * @param grantBean      授权对象
     * @param requestContext 当前用户
     */
    @Override
    @Transactional
    public Result grantService(Long id, GrantBean grantBean, RequestContext requestContext) {
        String path = ServiceTenant.class.getSimpleName() + "_" + requestContext.getTarget();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Tenant tenant = smsTenantService.detail(id);
            Assert.notNull(tenant, "租户不存在");
            List<ServiceBean> services = grantBean.getServices();
            //获取需要授权的服务id集合
            Set<Long> serviceIds = services.stream().map(ServiceBean::getServiceId).collect(Collectors.toSet());
            List<MetaBean> mates = grantBean.getMetas();
            //不开启配额 只处理授权
            if (!grantBean.getQuotaAble()) {
                Result grantResult = grantTenantService(id, serviceIds);
                if (grantResult.isFailed()) {
                    return grantResult;
                }
            } else {
                //配额处理 资源配额加入服务配额中
                for (MetaBean mate : mates) {
                    services.add(new ServiceBean(null, mate.getCode(), mate.getQuota(), mate.getUnit()));
                }
                grantTenantQuota(id, services);
                //处理授权信息
                Result grantResult = grantTenantService(id, serviceIds);
                if (grantResult.isFailed()) {
                    return grantResult;
                }
            }
            //配额开启设置变化则需要更新租户信息
            if (tenant.getQuotaAble() ^ grantBean.getQuotaAble()) {
                tenant.setQuotaAble(grantBean.getQuotaAble());
                Result result = smsTenantService.update(tenant);
                Assert.isTrue(result.isFailed(), "更新租户授权配置失败");
            }
            return new Result(true, "授权成功");
        }
    }

    private void grantTenantQuota(Long id, List<ServiceBean> services) {
        //查询租户已经有的配额
        Map<String, ResourceQuota> tenantQuotaMap = quotaRepository.list(id, null, null).stream()
                .collect(Collectors.toMap(ResourceQuota::getCode, Function.identity()));
        //特殊处理 避免配额报错
        services.removeIf(serviceBean -> !StringUtils.hasText(serviceBean.getUnit()) || null == serviceBean.getQuota());
        //授权配额的编码
        List<String> quotaCodes = services.stream().map(ServiceBean::getCode).collect(Collectors.toList());
        //判断配额是否能否被回收
        checkServiceQuota(quotaCodes, tenantQuotaMap);
        List<ResourceQuota> addQuotas = new ArrayList<>();
        List<ResourceQuota> updateQuotas = new ArrayList<>();
        List<String> removeQuotaCodes = new ArrayList<>();
        for (Map.Entry<String, ResourceQuota> entry : tenantQuotaMap.entrySet()) {
            String quotaCode = entry.getKey();
            //旧配额不存在新配额code集合 说明配额被回收
            if (!quotaCodes.contains(quotaCode)) {
                removeQuotaCodes.add(quotaCode);
            }
        }
        for (ServiceBean service : services) {
            String code = service.getCode();
            ResourceQuota quota = tenantQuotaMap.get(code);
            //有单位的才处理配额
            if (!StringUtils.hasText(service.getUnit())) {
                continue;
            }
            //存在旧配额则加入更新集合
            if (quota != null) {
                quota.setQuota(service.getQuota());
                updateQuotas.add(quota);
            } else {
                //不存在旧配额则加入新增配额集合
                ResourceQuota resourceQuota = new ResourceQuota(id, service.getQuota(), 0L, service.getUnit(),
                        service.getCode(), null);
                //有服务id则是服务配额
                if (service.getServiceId() != null) {
                    resourceQuota.setCategory(ResourceQuota.Category.Service);
                } else {
                    //无服务id则是基础配额
                    resourceQuota.setCategory(ResourceQuota.Category.Meta);
                }
                addQuotas.add(resourceQuota);
            }
        }
        //处理删除回收的配额
        if (!removeQuotaCodes.isEmpty()) {
            quotaRepository.delete(removeQuotaCodes);
        }
        //处理新增的配额
        if (!addQuotas.isEmpty()) {
            addQuotas.forEach(quotaRepository::save);
        }
        //处理更新的配额
        if (!updateQuotas.isEmpty()) {
            updateQuotas.forEach(quotaRepository::update);
        }
    }

    private Result grantTenantService(Long id, Set<Long> serviceIds) {
        try {
            List<ServiceTenant> serviceTenants = new ArrayList<>();
            List<Favorite> favorites = new ArrayList<>();
            for (Long serviceId : serviceIds) {
                ServiceTenant serviceTenant = new ServiceTenant(serviceId, id);
                serviceTenants.add(serviceTenant);
                Favorite favorite = new Favorite(null, Favorite.Category.Tenant, id, serviceId, 0);
                favorites.add(favorite);
            }
            // 先解除租户原有的服务授权
            this.cloudServiceRepository.deleteTenant(id, null);
            // 解除租户原有的收藏夹
            this.tenantService.cancelFavorite(id, null);
            //添加新的服务授权和收藏夹
            // 绑定当前授权的服务
            serviceTenants.forEach(serviceTenantRepository::save);
            if (!favorites.isEmpty()) {
                // 绑定当前授权的服务
                smsTenantService.addFavorites(favorites);
            }
            return new Result(true, "授权成功");
        } catch (Exception e) {
            log.error("grant tenant service error,", e);
            return new Result(false, "授权租户服务失败:", e.getMessage());
        }
    }

    /**
     * 校验服务配额能否被回收（有配额使用不能回收）
     *
     * @param quotaCodes 授权配额编码集合
     * @param quotaMap   已经授权配额集合
     */
    private void checkServiceQuota(List<String> quotaCodes, Map<String, ResourceQuota> quotaMap) {
        for (Map.Entry<String, ResourceQuota> entry : quotaMap.entrySet()) {
            String code = entry.getKey();
            //授权配额编码不包含原有配额编码(配额被移除了)
            if (!quotaCodes.contains(code)) {
                ResourceQuota quota = entry.getValue();
                // if (quota.getUsed() > 0) {
                if (quota.getUsed() > 0 && quota.getName() != null) {
                    throw new RuntimeException(quota.getName() + "配额存在使用,无法回收");
                }
            }
        }
    }

    /**
     * 服务授权多个租户
     *
     * @param serviceId      授权服务id
     * @param tenantIds      租户id
     * @param requestContext 请求对象
     * @return 结果
     */
    @Override
    @Transactional
    public Result grantTenant(Long serviceId, List<Long> tenantIds, RequestContext requestContext) {
        if (null == serviceId) {
            return new Result(false, "服务为空，授权失败");
        }
        String path = ServiceTenant.class.getSimpleName() + "_" + requestContext.getTarget();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if (ListTool.isEmpty(tenantIds)) {
                // 如果租户id集合为空，则取消所有租户该服务及配额
                List<ResourceQuota> resourceQuotas = quotaRepository.listServiceQuota(serviceId);
                // 有配额使用则无法取消服务授权
                if (resourceQuotas.stream().anyMatch(resourceQuota -> resourceQuota.getUsed() > 0)) {
                    return new Result(false, "授权失败,服务存在租户已使用配额，不能被解除");
                }
                // 取消服务授权
                this.cloudServiceRepository.deleteTenant(null, serviceId);
                // 取消租户服务收藏
                tenantService.cancelFavorite(null, serviceId);
                // 删除配额
                CloudService cloudService = cloudServiceRepository.query(serviceId);
                resourceQuotaService
                        .delete(null, ResourceQuota.Category.Service.name(), cloudService.getCode(), requestContext);
            } else {
                // 获取该服务所有已经存在租户服务授权关系
                List<ServiceTenant> tenants = serviceTenantRepository.listTenant(serviceId);
                // 用于接收当前已授权该服务的租户ID集合
                List<Long> currentTenantIds = Lists.newArrayList();
                // 用于接收需要解除该服务的租户id集合
                List<Long> removeTenantIds = Lists.newArrayList();
                for (ServiceTenant tenant : tenants) {
                    // 当前服务授权集合添加已经授权的租户id
                    currentTenantIds.add(tenant.getTenantId());
                    // 如果当前租户id集合中没有已授权租户id，则是需要解除授权的租户，把其id添加到解除服务租户id集合中
                    if (!tenantIds.contains(tenant.getTenantId())) {
                        removeTenantIds.add(tenant.getTenantId());
                    }
                }
                // 用于处理需要增加服务授权的租户id集合
                List<Long> addTenantIds = Lists.newArrayList();
                for (Long tenantId : tenantIds) {
                    // 已经授权该服务的租户id集合不包含当前租户id，则是需要新增授权的租户，添加的增加服务授权的租户id集合
                    if (!currentTenantIds.contains(tenantId)) {
                        addTenantIds.add(tenantId);
                    }
                }
                // 用于处理新增授权服务租户和服务关联的集合
                List<ServiceTenant> addTenants = Lists.newArrayList();
                // 用于处理新增收藏的集合
                List<Favorite> addFavorites = Lists.newArrayList();
                // 用于处理增加配额的集合
                List<ResourceQuota> addQuota = Lists.newArrayList();
                // 遍历新增服务的租户id集合，处理相关新增操作需要的数据集合
                if (!addTenantIds.isEmpty()) {
                    List<Long> quotaAbleTenants =
                            tenantService.listByIds(addTenantIds, "NORMAL").stream().filter(Tenant::getQuotaAble).map(Tenant::getId)
                                    .collect(Collectors.toList());
                    for (Long tenantId : addTenantIds) {
                        // 构建新增租户和服务关联对象集合及新增收藏集合
                        addTenants.add(new ServiceTenant(serviceId, tenantId));
                        addFavorites.add(new Favorite(null, Favorite.Category.Tenant, tenantId, serviceId, 0));
                        CloudService service = cloudServiceRepository.query(serviceId);
                        // 处理需要新增的服务配额
                        if (StringUtils.hasText(service.getUnit()) && quotaAbleTenants.contains(tenantId)) {
                            addQuota.add(new ResourceQuota(tenantId, 0L, 0L, service.getUnit(), service.getCode(),
                                    ResourceQuota.Category.Service));
                        }
                    }
                }
                if (!ListTool.isEmpty(removeTenantIds)) {
                    // 遍历解除服务的租户id集合，处理相关解除服务操作
                    for (Long tenantId : removeTenantIds) {
                        ResourceQuota quota = quotaRepository.query(tenantId, serviceId);
                        // 判断配额是否可以解除
                        if (quota != null && quota.getUsed() > 0) {
                            return new Result(false, "授权失败,租户存在服务已使用配额，无法解除");
                        }
                        // 解除该租户的授权服务
                        this.cloudServiceRepository.deleteTenant(tenantId, serviceId);
                        // 解除该租户的服务收藏
                        tenantService.cancelFavorite(tenantId, serviceId);
                        // 删除配额信息
                        CloudService cloudService = cloudServiceRepository.query(serviceId);
                        resourceQuotaService
                                .delete(tenantId, ResourceQuota.Category.Service.name(), cloudService.getCode(),
                                        requestContext);
                    }
                }
                if (!ListTool.isEmpty(addTenants)) {
                    // 保存租户授权关系
                    addTenants.forEach(serviceTenantRepository::save);
                    // 批量处理添加租户收藏夹
                    tenantService.addFavorites(addFavorites);
                    // 处理新增配额
                    if (!addQuota.isEmpty()) {
                        Map<Long, List<ResourceQuota>> addQuotaMap =
                                addQuota.stream().collect(Collectors.groupingBy(ResourceQuota::getTarget));
                        for (Map.Entry<Long, List<ResourceQuota>> entry : addQuotaMap.entrySet()) {
                            Long tenantId = entry.getKey();
                            List<ResourceQuota> quotas = entry.getValue();
                            resourceQuotaService.assign(tenantId, quotas, requestContext);
                        }
                    }
                }
            }
        }
        return new Result(true, "授权成功");
    }

    /**
     * 解除租户授权服务
     *
     * @param id             租户id
     * @param requestContext 请求对象
     * @return 结果
     */
    @Override
    @Transactional
    public Result deleteByTenant(Long id, RequestContext requestContext) {
        String path = ServiceTenant.class.getSimpleName() + "_" + requestContext.getTarget();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 判断租户下服务配额是否使用，有使用无法删除
            List<ResourceQuota> resourceQuotas = quotaRepository.listByTenantId(id);
            if (resourceQuotas.stream().anyMatch(resourceQuota -> resourceQuota.getUsed() > 0)) {
                return new Result(false, "删除租户服务关系失败,存在服务配额已使用。");
            }
            // 解绑租户服务授权
            this.cloudServiceRepository.deleteByTenant(id);
            // 取消租户服务收藏夹
            tenantService.cancelFavorite(id, null);
            // 删除租户配额
            resourceQuotaService.delete(id, ResourceQuota.Category.Service.name(), null, requestContext);
        }
        return new Result(true, "删除租户服务关系成功");
    }

    @Override
    public GeneralResult<List<CloudService>> listByTenant(String code, String suffix, RequestContext context) {
        try {
            List<CloudService> services = new ArrayList<>();
            if (RequestContext.Catalog.Tenant.equals(context.getCatalog())) {
                ServiceCatalog catalog = catalogRepository.getByCode(code);
                Assert.notNull(catalog, "未找到服务目录");
                List<ServiceTenant> serviceTenants = serviceTenantRepository.listByTenantId(context.getTenant());
                Assert.notEmpty(serviceTenants, "当前租户未授权任何云服务");
                for (ServiceTenant serviceTenant : serviceTenants) {
                    CloudService service = cloudServiceRepository.query(serviceTenant.getServiceId());
                    if (null != service && catalog.getId().equals(service.getCatalogId())) {
                        if (StringUtils.hasText(suffix) && !service.getCode().contains(suffix)) {
                            continue;
                        }
                        if ("vmware.storage.disk".equalsIgnoreCase(service.getCode())) {
                            continue;
                        }
                        if ("cnware.standard.volume".equalsIgnoreCase(service.getCode())) {
                            continue;
                        }
                        services.add(service);
                    }
                }
            } else {
                ServiceCatalog catalog = catalogRepository.getByCode(code);
                List<CloudService> cloudServices = smsUserService.getUserServices(context.getApiKey());
                services = cloudServices.stream().filter(cloudService -> {
                            if (catalog.getId().equals(cloudService.getCatalogId())) {
                                return !StringUtils.hasText(suffix) || cloudService.getCode().contains(suffix);
                            }
                            return false;
                        }).filter(cloudService -> !"vmware.storage.disk".equalsIgnoreCase(cloudService.getCode()))
                        .filter(cloudService -> !"cnware.standard.volume".equalsIgnoreCase(cloudService.getCode()))
                        .collect(Collectors.toList());
            }
            return new GeneralResult<>(true, services, "查询成功");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }
}
