package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudVendorAccount;
import com.bocloud.cos.enums.BocloudStatus;
import com.bocloud.cos.intf.VendorAccountService;
import com.bocloud.cos.provider.ProviderRegister;
import com.bocloud.cos.repository.VendorAccountRepository;
import com.google.common.collect.Maps;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.KeyGenerator;
import com.megatron.common.utils.ListTool;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author lym
 * @since 2022-06-16
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class VendorAccountServiceImpl implements VendorAccountService {

    private final VendorAccountRepository vendorAccountRepository;
    private final ProviderRegister providerRegister;

    @Override
    public Result create(CloudVendorAccount vendorAccount) throws Exception {
        vendorAccount.setAuthentication(StringEscapeUtils.unescapeJava(vendorAccount.getAuthentication()));
        JSONObject authentication = JSON.parseObject(vendorAccount.getAuthentication());
        if (authentication.containsKey("accessKey")) {
            String accessKey = new AESEncryptor().decrypt(authentication.getString("accessKey").trim(), null);
            accessKey = new DESEncryptor().encrypt(accessKey, "BocloudCMP");
            authentication.put("accessKey", accessKey);
        }
        if (authentication.containsKey("secretKey")) {
            String secretKey = new AESEncryptor().decrypt(authentication.getString("secretKey").trim(), null);
            secretKey = new DESEncryptor().encrypt(secretKey, "BocloudCMP");
            authentication.put("secretKey", secretKey);
        }
        List<String> currentRegions = JSONObject.parseArray(authentication.getString("regions"), String.class);
        if (!ListTool.isEmpty(currentRegions)) {
            authentication.put("regionId", currentRegions.get(0));
        }
        vendorAccount.setUuid(KeyGenerator.uuid());
        vendorAccount.setStatus(BocloudStatus.Basic.NORMAL.name());
        vendorAccount.setAuthentication(JSON.toJSONString(authentication));
        vendorAccountRepository.save(vendorAccount);
        register();
        return new Result(true, "新增成功");
    }

    @Override
    public Result modify(CloudVendorAccount vendorAccount) throws Exception {
        List<CloudVendorAccount> accounts = vendorAccountRepository.listById(vendorAccount.getId());
        if (CollectionUtils.isEmpty(accounts)) {
            return new Result(false, "编辑失败：数据不存在");
        }
        vendorAccount.setAuthentication(StringEscapeUtils.unescapeJava(vendorAccount.getAuthentication()));
        JSONObject authentication = JSON.parseObject(vendorAccount.getAuthentication());
        if (authentication.containsKey("accessKey")) {
            String accessKey = new AESEncryptor().decrypt(authentication.getString("accessKey").trim(), null);
            accessKey = new DESEncryptor().encrypt(accessKey, "BocloudCMP");
            authentication.put("accessKey", accessKey);
        }
        if (authentication.containsKey("secretKey")) {
            String secretKey = new AESEncryptor().decrypt(authentication.getString("secretKey").trim(), null);
            secretKey = new DESEncryptor().encrypt(secretKey, "BocloudCMP");
            authentication.put("secretKey", secretKey);
        }
        List<String> currentRegions = JSONObject.parseArray(authentication.getString("regions"), String.class);
        if (!ListTool.isEmpty(currentRegions)) {
            authentication.put("regionId", currentRegions.get(0));
        }
        CloudVendorAccount account = accounts.stream().findFirst().get();
        account.setVendorName(vendorAccount.getVendorName());
        account.setVendorType(vendorAccount.getVendorType());
        account.setAuthentication(vendorAccount.getAuthentication());
        account.setRemark(vendorAccount.getRemark());
        account.setAuthentication(JSON.toJSONString(authentication));
        vendorAccountRepository.update(account);
        register();
        return new Result(true, "编辑成功");
    }

    @Override
    public GeneralResult<GridBean<CloudVendorAccount>> list(Pager pager) {
        Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
        sorter.put("gmtCreate", Common.ONE);
        int total = vendorAccountRepository.count(pager.getParams());
        List<CloudVendorAccount> list = vendorAccountRepository.list(pager.getPage(), pager.getRows(), pager.getParams(), sorter);
        list.forEach(account -> {
            JSONObject authentication = JSON.parseObject(account.getAuthentication());
            if (authentication.containsKey("accessKey")) {
                String accessKey = new DESEncryptor().encrypt(authentication.getString("accessKey").trim(), "BocloudCMP");
                accessKey = new AESEncryptor().encrypt(accessKey, null);
                authentication.put("accessKey", accessKey);
            }
            if (authentication.containsKey("secretKey")) {
                String secretKey = new DESEncryptor().encrypt(authentication.getString("secretKey").trim(), "BocloudCMP");
                secretKey = new AESEncryptor().encrypt(secretKey, null);
                authentication.put("secretKey", secretKey);
            }
            account.setAuthentication(JSON.toJSONString(authentication));
        });
        GridBean<CloudVendorAccount> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        return new GeneralResult<>(true, gridBean, "查询成功");
    }

    @Override
    public Result remove(Long id, RequestContext context) {
        vendorAccountRepository.remove(id);
        register();
        return new GeneralResult<>(true, "删除成功");
    }

    @Override
    public GeneralResult<CloudVendorAccount> detail(Long id) {
        if (null == id || 0L == id) {
            return new GeneralResult<>(false, "参数无效");
        }
        List<CloudVendorAccount> list = vendorAccountRepository.listById(id);
        if (CollectionUtils.isEmpty(list)) {
            return new GeneralResult<>(false, "数据不存在");
        }
        return new GeneralResult<>(true, list.get(0), "查询成功");
    }

    @Override
    public Result importAccount(List<CloudVendorAccount> accounts) {
        try {
            accounts.forEach(vendorAccountRepository::save);
            register();
        } catch (Exception e) {
            return new Result(false, String.format("导入失败：%s", e.getMessage()));
        }
        return new Result(true, "导入成功");
    }

    @Override
    public GeneralResult<List<String>> queryVendorTypes() {
        List<CloudVendorAccount> accounts = vendorAccountRepository.queryVendorTypes();
        if (CollectionUtils.isEmpty(accounts)) {
            return new GeneralResult<>(true, new ArrayList<>(), "查询成功");
        }
        List<String> types = accounts.stream().map(CloudVendorAccount::getVendorType).collect(Collectors.toList());
        return new GeneralResult<>(true, types, "查询成功");
    }

    private void register() {
        try {
            List<CloudVendorAccount> cloudVendorAccounts = vendorAccountRepository.listById(null);
            providerRegister.updateProvider(cloudVendorAccounts);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
