package com.bocloud.cos.service.assess;

import com.bocloud.cos.entity.assess.AssessMetricClassify;
import com.bocloud.cos.repository.assess.AssessMetricClassifyRepository;
import com.megatron.common.model.RequestContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 考核指标分类业务逻辑处理
 *
 * @author lym
 */
@Slf4j
@Service
public class AssessMetricClassifyService {

    @Autowired
    private AssessMetricClassifyRepository assessCategoryRepository;

    /**
     * Retrieves a list of AssessCategory objects.
     *
     * @return The list of AssessCategory objects.
     */
    public List<AssessMetricClassify> query() {
        List<AssessMetricClassify> assessCategories = assessCategoryRepository.queryAll();
        if (assessCategories.isEmpty()) {
            return Collections.emptyList();
        }

        Long minParentId = assessCategories.stream().map(AssessMetricClassify::getParentId).min(Long::compareTo).get();
        Map<Long, List<AssessMetricClassify>> categoryMap = assessCategories.stream().collect(Collectors.groupingBy(AssessMetricClassify::getParentId));

        // Populate the children of each AssessCategory object
        populatesChildren(assessCategories, categoryMap);

        return assessCategories.stream()
                .filter(metricClassify -> minParentId.equals(metricClassify.getParentId()))
                .collect(Collectors.toList());
    }

    /**
     * Recursively queries the assess categories and populates the children based on the category map.
     *
     * @param assessCategories The list of assess categories to query.
     * @param categoryMap      The map of category IDs to their corresponding child categories.
     */
    private void populatesChildren(List<AssessMetricClassify> assessCategories, Map<Long, List<AssessMetricClassify>> categoryMap) {
        assessCategories.forEach(metricClassify -> {
            List<AssessMetricClassify> children = categoryMap.getOrDefault(metricClassify.getId(), Collections.emptyList());
            metricClassify.setChildren(children);
            if (!children.isEmpty()) {
                populatesChildren(children, categoryMap);
            }
        });
    }

    public void save(AssessMetricClassify metricClassify, RequestContext context) {
        saveOf(metricClassify, context);
        assessCategoryRepository.save(metricClassify);
    }

    public void update(AssessMetricClassify metricClassify, RequestContext context) {
        AssessMetricClassify ac = getAssessCategory(metricClassify.getId());
        ac.setName(metricClassify.getName());
        updateOf(ac, context);
        assessCategoryRepository.update(ac);
    }

    public void remove(Long id, RequestContext context) {
        AssessMetricClassify metricClassify = getAssessCategory(id);
        metricClassify.setDeleted(true);
        updateOf(metricClassify, context);
        assessCategoryRepository.update(metricClassify);
    }

    private AssessMetricClassify getAssessCategory(Long id) {
        AssessMetricClassify metricClassify = assessCategoryRepository.get(AssessMetricClassify.class, id);
        Assert.notNull(metricClassify, "该考核分类不存在");
        return metricClassify;
    }

    private void saveOf(AssessMetricClassify metricClassify, RequestContext context) {
        metricClassify.setCreatorId(context.getTarget());
        metricClassify.setGmtCreate(new Date());
    }

    private void updateOf(AssessMetricClassify metricClassify, RequestContext context) {
        metricClassify.setMenderId(context.getTarget());
        metricClassify.setGmtModify(new Date());
    }
}
