package com.bocloud.cos.service.assess;

import com.bocloud.cos.entity.assess.*;
import com.bocloud.cos.enums.AssessRecordStatusEnum;
import com.bocloud.cos.enums.AssessTaskStatusEnum;
import com.bocloud.cos.repository.assess.*;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 考核任务业务逻辑处理
 *
 * @author lym
 */
@Slf4j
@Service
public class AssessTaskService {

    @Autowired
    AssessTaskRepository taskRepository;
    @Autowired
    AssessTaskMetricRepository taskMetricRepository;
    @Autowired
    AssessObjectRepository objectRepository;
    @Autowired
    AssessRecordRepository recordRepository;
    @Autowired
    AssessRecordMetricRepository recordMetricRepository;

    public GridBean<AssessTask> list(Pager pager, RequestContext context) {
        pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
        pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
        int total = taskRepository.count(pager.getParams());
        List<AssessTask> list = taskRepository.list(pager.getPage(), pager.getRows(), pager.getSorter(), pager.getParams());
        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }

    @Transactional(rollbackFor = Exception.class)
    public void save(AssessTask task, RequestContext context) {

        task.setIsTemplate(task.getIsTemplate() != null ? task.getIsTemplate() : false);
        task.setStatus(AssessTaskStatusEnum.CREATEING.getCode());
        saveOf(task, context);
        taskRepository.save(task);

        if (!task.getIsTemplate()) {
            Assert.notEmpty(task.getObjectList(), "请选择考核对象");
            batchSaveObjectDetail(task.getId(), task.getObjectList());
        }

        if (null != task.getTemplateId()) {
            List<AssessTaskMetric> assessTaskMetrics = taskMetricRepository.listByTaskId(task.getTemplateId());
            assessTaskMetrics.forEach(assessTaskMetric -> {
                assessTaskMetric.setId(null);
                assessTaskMetric.setTaskId(task.getId());
            });
            taskMetricRepository.batchSave(assessTaskMetrics, assessTaskMetrics.size());
        }
    }

    public void update(AssessTask task, RequestContext context) {
        Assert.notNull(task.getId(), "考核任务id不能为空");
        AssessTask assessTask = taskRepository.get(AssessTask.class, task.getId());
        if (StringUtils.hasText(task.getName())) {
            assessTask.setName(task.getName());
        }
        if (task.getStandardId() != null) {
            assessTask.setStandardId(task.getStandardId());
        }
        if (StringUtils.hasText(task.getCycleType())) {
            assessTask.setCycleType(task.getCycleType());
        }
        if (StringUtils.hasText(task.getCycleValue())) {
            assessTask.setCycleValue(task.getCycleValue());
        }
        assessTask.setRemark(task.getRemark());
        assessTask.setObjectPath(task.getObjectPath());
        updateOf(assessTask, context);
        boolean update = taskRepository.update(assessTask);
        if (update && !CollectionUtils.isEmpty(task.getObjectList())) {
            objectRepository.deleteByTaskId(task.getId());
            batchSaveObjectDetail(task.getId(), task.getObjectList());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void remove(Long id) {
        objectRepository.deleteByTaskId(id);
        taskRepository.deleteById(id);
    }

    private void batchSaveObjectDetail(Long taskId, List<AssessObject> objectList) {
        objectList.forEach(object -> {
            object.setTaskId(taskId);
        });
        objectRepository.batchSave(objectList, objectList.size());
    }

    private void saveOf(AssessTask task, RequestContext context) {
        task.setCreatorId(context.getTarget());
        task.setGmtCreate(new Date());
    }

    private void updateOf(AssessTask task, RequestContext context) {
        task.setMenderId(context.getTarget());
        task.setGmtModify(new Date());
    }

    public GridBean<AssessObject> listAssessObject(Pager pager, Long taskId) {
        pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
        pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
        int total = objectRepository.count(taskId);
        List<AssessObject> list = objectRepository.list(pager.getPage(), pager.getRows(), pager.getSorter(), taskId);
        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }

    public AssessTask detail(Long taskId) {
        AssessTask detail = taskRepository.detail(taskId);
        detail.setObjectList(objectRepository.queryByTaskId(taskId));
        return detail;
    }

    @Transactional(rollbackFor = Exception.class)
    public void taskPublish(Long taskId, RequestContext context) {
        AssessTask task = taskRepository.get(AssessTask.class, taskId);
        if (!AssessTaskStatusEnum.WAIT_PUBLISH.getCode().equals(task.getStatus())) {
            throw new IllegalArgumentException("非待发布状态下不能发布");
        }

        // 查询任务指标
        List<AssessTaskMetric> assessTaskMetrics = taskMetricRepository.listByTaskId(taskId);

        // 生成考核记录
        List<AssessObject> assessObjects = objectRepository.queryByTaskId(taskId);
        assessObjects.forEach(assessObject -> {
            AssessRecord assessRecord = new AssessRecord();
            assessRecord.setTaskId(taskId);
            assessRecord.setAssessObjId(assessObject.getId());
            assessRecord.setStatus(AssessRecordStatusEnum.RECEIVE_ASSESS.getCode());
            assessRecord.setExecute(false);
            assessRecord.setExecuteScore(0.0);
            assessRecord.setQualifiedCount(0);
            assessRecord.setUnqualifiedCount(0);
            assessRecord.setCreatorId(context.getTarget());
            assessRecord.setGmtCreate(new Date());
            assessRecord.setRemark(task.getRemark());
            recordRepository.save(assessRecord);

            /* 生成考核记录与考核指标细则 */
            assessTaskMetrics.forEach(taskMetric -> {
                // 生成考核记录与考核指标细则
                AssessRecordMetric recordMetric = new AssessRecordMetric();
                recordMetric.setRecordId(assessRecord.getId());
                recordMetric.setMetricRuleId(taskMetric.getMetricRuleId());
                recordMetric.setScore(0.0);
                recordMetric.setQualified(false);
                recordMetric.setMaxScore(taskMetric.getMaxScore());
                recordMetric.setWeight(taskMetric.getWeight());
                recordMetric.setCreatorId(context.getTarget());
                recordMetric.setGmtCreate(new Date());
                recordMetricRepository.save(recordMetric);
            });
        });

        // 任务发布后，修改任务状态为考核中
        task.setStatus(AssessTaskStatusEnum.ASSESSING.getCode());
        updateOf(task, context);
        taskRepository.update(task);
    }

    public void convertToTemplate(Long taskId, RequestContext context) {
        AssessTask assessTask = taskRepository.get(AssessTask.class, taskId);
        assessTask.setId(null);
        assessTask.setIsTemplate(true);
        saveOf(assessTask, context);
        taskRepository.save(assessTask);

        List<AssessObject> assessObjects = objectRepository.queryByTaskId(taskId);
        assessObjects.forEach(assessObject -> {
            assessObject.setTaskId(assessTask.getId());
        });
        objectRepository.batchSave(assessObjects, assessObjects.size());

        List<AssessTaskMetric> assessTaskMetrics = taskMetricRepository.listByTaskId(taskId);
        if (CollectionUtils.isEmpty(assessTaskMetrics)) {
            return;
        }
        assessTaskMetrics.forEach(assessTaskMetric -> {
            assessTaskMetric.setTaskId(assessTask.getId());
        });
        taskMetricRepository.batchSave(assessTaskMetrics, assessTaskMetrics.size());
    }
}
