package com.bocloud.cos.service.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cos.entity.internel.Dictionary;
import com.bocloud.cos.entity.internel.Favorite;
import com.bocloud.cos.entity.internel.Tenant;
import com.google.common.collect.Maps;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * soc服务租户内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月4日
 */
@Component
public class SmsTenantService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsTenantService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private static final Service SERVICE = Service.create("sms");
    private final Map<String, Object> headers = new HashMap<>(4);

    /**
     * 获取租户详情
     *
     * @param id 租户id
     * @return
     */
    public Tenant detail(Long id) {
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/tenants/" + id, HttpMethod.GET, headers, null);
        GeneralResult result = remote.invoke();
        if (result.isSuccess() && result.getData() != null) {
            return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), Tenant.class);
        } else {
            return null;
        }
    }

    /**
     * 分页获取用户列表
     *
     * @param pager 分页对象
     * @return
     */
    public List<Tenant> list(Pager pager) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants",
                HttpMethod.GET, headers, pager.toMap());
        GeneralResult<?> result = remoteService.invoke();
        return ((JSONObject) result.getData()).getJSONArray(Common.ROWS).toJavaList(Tenant.class);
    }

    /**
     * 根据id集合获取对应用户集合
     *
     * @param ids 用户id集合
     * @return
     */
    public List<Tenant> listByIds(List<Long> ids, String status) {
        List<Param> params = new ArrayList<>();
        if (ids.isEmpty()) {
            ids.add(-1L);
        }
        params.add(new Param(MapTools.simpleMap("id", JSONObject.toJSONString(ids).substring(1, JSONObject.toJSONString(ids).length() - 1)), Sign.IN));
        if (StringUtils.isNotBlank(status)) {
            params.add(new Param(MapTools.simpleMap("status", status), Sign.EQ));
        }
        return list(new Pager(1, Integer.MAX_VALUE, params, null));
    }

    /**
     * 添加服务收藏
     *
     * @param favorites
     * @return
     */
    public boolean addFavorites(List<Favorite> favorites) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/favorites", HttpMethod.POST, headers, favorites);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke().isSuccess();
    }

    /**
     * 取消收藏
     *
     * @param tenantId
     * @param itemId
     * @return
     */
    public boolean cancelFavorite(Long tenantId, Long itemId) {
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        params.put("serviceId", itemId);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/favorites", HttpMethod.DELETE, headers, params);
        remoteService.setDataFormat(PostDataFormat.RAW);
        GeneralResult<?> invoke = remoteService.invoke();
        return invoke.isSuccess();
    }

    public Result update(Tenant tenant) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/" + tenant.getId(),
                HttpMethod.PUT, headers, tenant);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
    }

    public GeneralResult<List<Tenant>> listAll(List<Param> paramList) {
        Map<String, Object> params = Maps.newHashMap();
        params.put(Common.PAGE, 1);
        params.put(Common.ROWS, Integer.MAX_VALUE);
        params.put(Common.SIMPLE, false);
        params.put(Common.PARAMS, JSONObject.toJSONString(paramList));
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants", HttpMethod.GET, headers, params);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        GridBean gridBean = JSONObject.parseObject(data, GridBean.class);
        return new GeneralResult<>(true,
                JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), Tenant.class), "success");
    }


    public Map<String, String> tenantProvince() {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/dictionaries", HttpMethod.GET, headers,
                MapTools.simpleMap("value", "TENANT_PROVINCE"));
        GeneralResult invoke = remoteService.invoke();
        Map<String, String> result = new HashMap<>();
        if (invoke.getData() != null) {
            List<Dictionary> dictionaries =
                    JSONArray.parseArray(JSONObject.toJSONString(invoke.getData()), Dictionary.class);
            for (Dictionary dictionary : dictionaries) {
                result.put(dictionary.getValue(), dictionary.getName());
            }
        }
        return result;
    }

    public GeneralResult listTenants(Pager pager) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants", HttpMethod.GET, headers, pager.toMap());
        return remoteService.invoke();
    }
}
